/**
 * 
 */
package org.springblade.cube.service;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.core.redis.cache.CacheKey;
import org.springblade.core.secure.BladeUser;
import org.springblade.cube.enums.ParamTypeEnum;
import org.springblade.cube.mapper.CubeGlobalParamMapper;
import org.springblade.cube.model.CubeGlobalParam;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import java.time.Duration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author wookvn
 *
 */
@Service
public class CubeGlobalParamService extends ServiceImpl<CubeGlobalParamMapper, CubeGlobalParam> {

    @Autowired
    private BladeRedis bladeRedis;

    final static String GLOBALPARAM_CAHCE_KEY_PREFIX = "bi:globalparam:";
    final static int GLOBALPARAM_CAHCE_TIMEOUT_SEC = 1 * 60 * 60;

    public List<CubeGlobalParam> listByTypeAndTenantIdFromCache(ParamTypeEnum paramType, String tenantId) {
        String cacheKey = GLOBALPARAM_CAHCE_KEY_PREFIX + paramType + ":" + tenantId;
        return bladeRedis.get(new CacheKey(cacheKey, Duration.ofSeconds(GLOBALPARAM_CAHCE_TIMEOUT_SEC)), () -> baseMapper.selectList(Wrappers.<CubeGlobalParam>lambdaQuery()
                .eq(CubeGlobalParam::getParamType, ParamTypeEnum.BIZ).eq(CubeGlobalParam::getTenantId, tenantId)));
    }

    public void refreshCache(ParamTypeEnum paramType, String tenantId) {
        String cacheKey = GLOBALPARAM_CAHCE_KEY_PREFIX + paramType + ":" + tenantId;
        bladeRedis.del(cacheKey);
    }

    /**
     * 获取系统默认全局变量
     */
    public Map<String, Object> getSysParams(DateFormatEnum dateFormat, BladeUser bladeUser) {
        Map<String, List<Object>> stringListMap = CubeConstants.mergeAllSysParams(bladeUser,dateFormat, new HashMap<>());
        Map<String,Object> result = new HashMap<>();
        if (CollectionUtil.isNotEmpty(stringListMap)) {
            for (Map.Entry<String, List<Object>> entry : stringListMap.entrySet()) {
                result.put(entry.getKey(),entry.getValue().get(0));
            }
        }
        return result;
    }
}
