/*
 * Copyright (c) 2018-2028, aokunsang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.cube.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springblade.bi.client.form.QueryCubeWhereInForm;
import org.springblade.bi.client.model.QueryCubeModel;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springframework.stereotype.Service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;

/**
 * 数据集简单查询 - 不用校验权限组等复杂操作，只需要拼接sql查询即可
 *
 * @author wookvn
 * @datetime 2024-10-22 16:40:39
 */
@Slf4j
@Service
@AllArgsConstructor
public class CubeSimpleQueryService {

    private final CubeDataService cubeDataService;

    private final IReportDatasourceService reportDatasourceService;

    private final CubeTableColumnService cubeTableColumnService;

    private final DynamicTableService dynamicTableService;

    /**
     * <p>数据集查询-wherein条件</p>
     * @author wookvn
     * @datetime 2024-10-22 16:34:48
     * @param queryCubeWhereInForm: 查询参数
     * @return org.springblade.bi.client.model.QueryCubeModel
     **/
    public QueryCubeModel listCubeWhereIn(QueryCubeWhereInForm queryCubeWhereInForm) {
        CubeData cubeData = cubeDataService.getById(queryCubeWhereInForm.getCubeId());
        if(cubeData == null) {
            throw new ServiceException("数据集不存在");
        }
        //构造或查询数据源
        ReportDatasource reportDatasource = null;
        if(StrUtil.isNotEmpty(queryCubeWhereInForm.getPoolName())) {
            reportDatasource = new ReportDatasource();
            reportDatasource.setPoolName(queryCubeWhereInForm.getPoolName());
        }else {
            reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
            if(reportDatasource == null) {
                throw new ServiceException("数据集引用的数据源不存在");
            }
        }
        //构造SQL
        List<CubeTableColumn> cubeTableColumnList = cubeTableColumnService.listByCubeId(queryCubeWhereInForm.getCubeId());
        //存储别名和ID的映射关系
        Map<Integer, String> fieldDict = new HashMap<>();
        List<String> selectColumnList = new ArrayList<>();
        //wherein条件的字段名
        String whereInColumnName = "";

        //需要查询的字段ID
        List<Integer> selectCubeColumnIdList = queryCubeWhereInForm.getSelectCubeColumnIdList();

        //初始化别名
        int index = 0;
        for(CubeTableColumn cubeTableColumn : cubeTableColumnList) {
            String columnAliasName = CubeConstants.COLUMN_ALIAS_NAME_PREFIX + (index++),
                   selectColumnSql = cubeTableColumn.getColumnSql() + " as \"" + columnAliasName + "\"";
            if(CollectionUtil.isEmpty(selectCubeColumnIdList) || selectCubeColumnIdList.stream().anyMatch(item -> cubeTableColumn.getId().intValue() == item)) {
                fieldDict.put(cubeTableColumn.getId(), columnAliasName);
                selectColumnList.add(selectColumnSql);
            }
            if(cubeTableColumn.getId().intValue() == queryCubeWhereInForm.getPrimaryKeyCubeColumnId()) {
                whereInColumnName = cubeTableColumn.getColumnSql();
            }
        }

        if(StrUtil.isBlank(whereInColumnName)) {
            log.info("未找到主键ID字段, queryCubeWhereInForm.getPrimaryKeyCubeColumnId():{}", queryCubeWhereInForm.getPrimaryKeyCubeColumnId());
            log.info("未找到主键ID字段, cubeTableColumnList:{}", cubeTableColumnList.toArray());
            throw new ServiceException("未查询到主键ID字段");
        }

        //拼接SQL和请求参数
        String querySql = String.format("select %s from %s where %s in (:param)",
                String.join(",", selectColumnList), cubeData.getQuerySql(), whereInColumnName);
        Map<String, Object> paramMap = MapUtil.of("param", queryCubeWhereInForm.getParamValue());

        //数据结果集
        List<Map<String, Object>> dataList = dynamicTableService.namedPreparedQuery(reportDatasource, querySql, paramMap);

        return new QueryCubeModel(dataList, fieldDict);
    }
}
