/**
 *
 */
package org.springblade.resouces;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import org.apache.commons.collections4.map.HashedMap;
import org.springblade.bi.client.DynamicDataSourceClient;
import org.springblade.bi.client.form.*;
import org.springblade.bi.client.model.PageModel;
import org.springblade.bi.client.model.QueryCubeModel;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.exception.RpcException;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.cube.dto.BrowserParam;
import org.springblade.cube.dto.CubeTableName;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeTableColumn;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.CubeSimpleQueryService;
import org.springblade.cube.service.CubeTableColumnService;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import cn.hutool.core.collection.CollectionUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 */
@RestController
@Slf4j
public class DynamicDataSourceResource implements DynamicDataSourceClient {

    @Autowired
    private DynamicTableService dynamicTableService;

    @Autowired
    private CubeSimpleQueryService cubeSimpleQueryService;

    @Autowired
    private CubeDataService cubeDataService;

    @Autowired
    private IReportDatasourceService reportDatasourceService;
    @Autowired
    private CubeTableColumnService cubeTableColumnService;

    @Override
    public List<JSONObject> list(BaseDsForm dsForm) {
        ReportDatasource reportDatasource = new ReportDatasource();
        reportDatasource.setPoolName(dsForm.getPoolName());
        return dynamicTableService.selectListForJSONObject(reportDatasource, dsForm.getSql());
    }

    @Override
    public PageModel<JSONObject> page(PageDsForm dsForm) {
        ReportDatasource reportDatasource = new ReportDatasource();
        reportDatasource.setPoolName(dsForm.getPoolName());
        Page<?> pageParam = new Page<>(dsForm.getCurrent(), dsForm.getSize());
        IPage<JSONObject> page = dynamicTableService.selectPageForJSONObject(pageParam, reportDatasource, dsForm.getSql());
        PageModel<JSONObject> pageResult = new PageModel<>();
        if (page != null && CollectionUtil.isNotEmpty(page.getRecords())) {
            pageResult.setRecords(page.getRecords());
            pageResult.setCurrent(page.getCurrent());
            pageResult.setPages(page.getPages());
            pageResult.setTotal(page.getTotal());
            pageResult.setSize(page.getSize());
        }
        return pageResult;
    }

    @Override
    public JSONObject getOne(BaseDsForm dsForm) {
        ReportDatasource reportDatasource = new ReportDatasource();
        reportDatasource.setPoolName(dsForm.getPoolName());
        return dynamicTableService.selectOneForJSONObject(reportDatasource, dsForm.getSql());
    }

    @Override
    public void execute(BaseDsForm dsForm) {
        try {
            ReportDatasource reportDatasource = new ReportDatasource();
            reportDatasource.setPoolName(dsForm.getPoolName());
            dynamicTableService.executeSql(reportDatasource, dsForm.getSql());
        } catch (Exception e) {
            throw new RpcException(e.getMessage(), e);
        }
    }

    @Override
    public JSONObject executeNew(BaseDsForm dsForm) {
        JSONObject jsonObject = new JSONObject();
        try {
            ReportDatasource reportDatasource = new ReportDatasource();
            reportDatasource.setPoolName(dsForm.getPoolName());
            dynamicTableService.executeSql(reportDatasource, dsForm.getSql());
            jsonObject.put("code", "200");
            return jsonObject;
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            jsonObject.put("code", "500");
            return jsonObject;
        }
    }

    @Override
    public Long insertReturnAutoIncrementIdValue(BaseDsForm dsForm) {
        try {
            ReportDatasource reportDatasource = new ReportDatasource();
            reportDatasource.setPoolName(dsForm.getPoolName());
            return dynamicTableService.insertReturnIdValue(reportDatasource, dsForm.getSql(), dsForm.getIdColumn());
        } catch (Exception e) {
            throw new RpcException(e.getMessage(), e);
        }
    }

    @Override
    public void preparedExecute(BaseDsForm dsForm) {
        try {
            ReportDatasource reportDatasource = new ReportDatasource();
            reportDatasource.setPoolName(dsForm.getPoolName());
            dynamicTableService.preparedExecute(reportDatasource, dsForm.getSql(), dsForm.getBatchArgs());
        } catch (Exception e) {
            throw new RpcException(e.getMessage(), e);
        }
    }

    @Override
    public List<Map<String, Object>> namedPreparedQueryList(QueryDsForm queryDsForm) {
        try {
            ReportDatasource reportDatasource = new ReportDatasource();
            reportDatasource.setPoolName(queryDsForm.getPoolName());
            Map<String, Object> paramMap = queryDsForm.getParamObj();
            String sql = queryDsForm.getSql();
            return dynamicTableService.namedPreparedQuery(reportDatasource, sql, paramMap);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RpcException(e.getMessage(), e);
        }
    }

    @Override
    public QueryCubeModel listCubeWhereIn(QueryCubeWhereInForm queryCubeWhereInForm) {
        try {
            return cubeSimpleQueryService.listCubeWhereIn(queryCubeWhereInForm);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new RpcException(e.getMessage(), e);
        }
    }


    @Override
    public boolean updateTable(UpdateTableForm updateTableForm) {
        Map<String, Object> setValueMap = new HashMap<>();
        StringBuffer sql  = new StringBuffer();
        CubeData cubeData = cubeDataService.getById(updateTableForm.getCubeId());
        ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
        if (cubeData == null) {
            throw new ServiceException("数据集不存在");
        }
        if (reportDatasource == null) {
            throw new ServiceException("数据源不存在");
        }
        List<CubeTableName> cubeTableNames = JSON.parseArray(cubeData.getCubeTableNames(), CubeTableName.class);
        sql.append("update "+cubeTableNames.get(0).getTableName() + " set ");
        //处理set语句
        List<CubeTableColumn> setColumns = cubeTableColumnService.listByIds(updateTableForm.getSetMap().keySet());
        Map<Integer, CubeTableColumn> setColumnMap = setColumns.stream().collect(Collectors.toMap(CubeTableColumn::getId, Function.identity()));
        List<String> setSqlList = new ArrayList<String>();
        for (Integer key : updateTableForm.getSetMap().keySet()) {
            CubeTableColumn cubeTableColumn = setColumnMap.get(key);
            setSqlList.add(cubeTableColumn.getColumnName()+" = :" + cubeTableColumn.getColumnName());
            setValueMap.put(cubeTableColumn.getColumnName(),updateTableForm.getSetMap().get(key));
        }
        sql.append(String.join(",", setSqlList));
        List<String> conditionSqlList = new ArrayList<String>();
        if (CollectionUtil.isNotEmpty(updateTableForm.getParamList())) {
            List<Integer> columnIds = updateTableForm.getParamList().stream().map(CubeBrowserSqlParam::getCubeColumnId).collect(Collectors.toList());
            List<CubeTableColumn> cubeTableColumns = cubeTableColumnService.listByIds(columnIds);
            Map<Integer, CubeTableColumn> columnMap = cubeTableColumns.stream().collect(Collectors.toMap(CubeTableColumn::getId, Function.identity()));
            //获取where 条件
            List<BrowserParam> browserParams = BeanUtil.copyToList(updateTableForm.getParamList(), BrowserParam.class);
            for (BrowserParam browserParam : browserParams) {
                CubeTableColumn cubeTableColumn = columnMap.get(browserParam.getCubeColumnId());
                String conditionSql = browserParam.getFunction().handle(cubeTableColumn.getColumnName(), cubeTableColumn.getViewDataType(), browserParam.getValues(), reportDatasource.getDsTypeSyl());
                if (StrUtil.isNotEmpty(conditionSql)) {
                    conditionSqlList.add(conditionSql);
                }
            }
        }
        if (CollectionUtil.isNotEmpty(conditionSqlList)){
            sql.append(" where "+ String.join(",", conditionSqlList));
        }
        dynamicTableService.namedPreparedExecute(reportDatasource,sql.toString(), Arrays.asList(setValueMap));
        return true;
    }

}