/*
 * Copyright (c) 2018-2028, aokunsang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.visual.controller;

import java.util.ArrayList;
import java.util.Base64;
import java.util.Comparator;
import java.util.List;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import cn.hutool.core.lang.Tuple;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.cube.dto.PreviewResult;
import org.springblade.cube.dto.wayneDiagramResult;
import org.springblade.visual.dto.ComponentDto;
import org.springblade.visual.service.component.VisualComponentSangjiService;
import org.springblade.visual.service.VisualComponentService;
import org.springblade.visual.service.component.VisualComponentHistogramService;
import org.springblade.visual.service.component.VisualComponentWeienService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson2.JSON;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * <p>特殊组件控制器(如：直方图，桑吉图等)</p>
 * @author  wookvn
 * @datetime 2024-10-12 11:27:15
 */
@RestController
@RequestMapping("visual/component")
@Api(value = "特殊组件", tags = "特殊组件接口")
public class VisualComponentSpecialController extends BladeController {

    @Autowired
    private VisualComponentService visualComponentService;

    @Autowired
    private VisualComponentHistogramService visualComponentHistogramService;

    @Autowired
    private VisualComponentSangjiService visualComponentSangjiService;

    @Autowired
    private VisualComponentWeienService visualComponentWeienService;

    final ThreadPoolExecutor sangjituThreadPool = new ThreadPoolExecutor(5, 10, 0, TimeUnit.SECONDS,
            new ArrayBlockingQueue<>(1000), ThreadUtil.createThreadFactory("sangjitu-threadpool-"));

    /**
     * @param value
     * @return
     * @throws Exception
     */
    @PostMapping("/histogram/preview")
    @ApiOperationSupport(order = 4)
    @ApiOperation(value = "直方图数据预览", notes = "传入ComponentDto")
    public R<PreviewResult> histogramPreview(@RequestBody String value) throws Exception {
        Tuple tuple = visualComponentHistogramService.histogramPreview(value);
        PreviewResult previewResult = tuple.get(0);
        String sql = tuple.get(1);
        return R.data(previewResult, visualComponentService.showSql(sql));
    }

    /**
     * 桑基图数据预览
     * @param value
     * @return
     * @throws Exception
     */
    @PostMapping("/special/preview")
    @ApiOperationSupport(order = 6)
    @ApiOperation(value = "桑基图数据预览", notes = "传入ComponentDto")
    public R<List<PreviewResult>> sangituPreview(@RequestBody String value) throws Exception {
        ComponentDto componentDto = visualComponentService.previewCheck(value);
        // 判断当前的查询是否需要补齐数据
        JSONArray jsonArray = JSONUtil.parseArray(componentDto.getColumnList());
        List<PreviewResult> list = new ArrayList<>();
        CountDownLatch latch = new CountDownLatch(jsonArray.size());
        StringBuffer sqls = new StringBuffer();
        BladeUser user = getUser();
        for (int i = 0; i < jsonArray.size(); i++) {
            final int index = i;
            Object obj = jsonArray.get(i);
            sangjituThreadPool.execute(() -> {
                ComponentDto dto = BeanUtil.copyProperties(componentDto, ComponentDto.class);
                dto.setColumnList(obj.toString());
                visualComponentSangjiService.previewData(obj, dto, list, sqls, user, Integer.valueOf(index));
                latch.countDown();
            });
        }
        latch.await();
        return R.data(list.stream().sorted(Comparator.comparing(PreviewResult::getOrder)).collect(Collectors.toList()), visualComponentService.showSql(sqls.toString()));
    }

    /**
     * 韦恩图查询
     *
     * @param value
     * @return
     * @throws Exception
     */
    @PostMapping("/wayne/preview")
    @ApiOperationSupport(order = 7)
    @ApiOperation(value = "韦恩图数据预览", notes = "传入ComponentDto")
    public R<List<wayneDiagramResult>> wayneDiagramPreview(@RequestBody String value) throws Exception {
        // 判断当前的查询是否需要补齐数据
        List<wayneDiagramResult> list = new ArrayList<>();
        StringBuffer sqls = new StringBuffer();
        visualComponentWeienService.previewWayneDiagram(value, list, sqls);
        // 查询韦恩图具体方法
        return R.data(list.stream().sorted(Comparator.comparing(wayneDiagramResult::getSize).reversed()).collect(Collectors.toList()), visualComponentService.showSql(sqls.toString()));
    }
}

