package org.springblade.visual.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.visual.dto.SubmitComponentTemplateParams;
import org.springblade.visual.dto.SubmitComponentTemplateReviewParams;
import org.springblade.visual.dto.UpdateComponentTemplateBasicInfoParams;
import org.springblade.visual.dto.VisualComponentTemplatePageParams;
import org.springblade.visual.entity.VisualComponentTemplate;
import org.springblade.visual.entity.VisualComponentTemplateCategory;
import org.springblade.visual.entity.VisualReviewLog;
import org.springblade.visual.service.VisualComponentTemplateCategoryService;
import org.springblade.visual.service.VisualComponentTemplateService;
import org.springblade.visual.service.VisualReviewLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 用户端组件模板
 */
@Api(value = "(BI)用户端组件模板", tags = "(BI)用户端组件模板")
@RestController
@RequestMapping("/visual/component/template")
public class VisualComponentTemplateController {

    @Autowired
    private VisualComponentTemplateCategoryService componentTemplateCategoryService;
    @Autowired
    private VisualReviewLogService visualReviewLogService;
    @Autowired
    private VisualComponentTemplateService visualComponentTemplateService;
    @Autowired
    private VisualComponentTemplateCategoryService visualComponentTemplateCategoryService;

    /**
     * 创建目录
     */
    @PostMapping("/createCategory")
    @ApiOperation("创建目录")
    public R<Boolean> createCategory(@RequestBody VisualComponentTemplateCategory category) {
        return R.data(componentTemplateCategoryService.createCategory(category));
    }

    /**
     * 修改分类
     *
     * @param category
     */
    @PostMapping("/editCategory")
    @ApiOperation("修改分类")
    public R<Boolean> editCategory(@RequestBody VisualComponentTemplateCategory category) {
        //判重
        List<VisualComponentTemplateCategory> list = componentTemplateCategoryService.lambdaQuery()
                .eq(VisualComponentTemplateCategory::getName, category.getName())
                .eq(VisualComponentTemplateCategory::getParentId,category.getParentId())
                .ne(category.getId() != null, VisualComponentTemplateCategory::getId, category.getId()).list();
        if (CollectionUtil.isNotEmpty(list)) return R.fail("名字重复");

        return R.data(componentTemplateCategoryService.editCategory(category));
    }

    /**
     * 删除分类
     */
    @PostMapping("/editCategory/{id}")
    @ApiOperation("删除分类")
    public R<Boolean> deleteCategory(@PathVariable Long id) {
        return R.data(componentTemplateCategoryService.deleteCategory(id));
    }

    /**
     * 查询所有目录
     * 然后通过查询到的数据的parent_id来构建一个树结构
     */
    @ApiOperation("查询所有目录")
    @GetMapping("/queryAllCategory")
    public R<List<VisualComponentTemplateCategory>> queryAllCategory(String name) {
        return R.data(componentTemplateCategoryService.queryAllCategory(name));
    }

    /**
     * 组件模板列表
     */
    @GetMapping("/page")
    @ApiOperation("组件模板列表")
    public R<IPage<VisualComponentTemplate>> page(VisualComponentTemplatePageParams params) {
        return R.data(visualComponentTemplateService.page(params));
    }


    @ApiOperation("获取审核记录")
    @GetMapping("/listReviewLogs/{id}")
    public R<List<VisualReviewLog>> listReviewLogs(@PathVariable Long id) {
        return R.data(visualReviewLogService.getBaseMapper().listByRelationIdAndType(id, 2));
    }


    @ApiOperation(value = "发起审核")
    @PostMapping("/submitReview")
    public R submitReview(@RequestBody SubmitComponentTemplateReviewParams params) {
        return R.status(visualComponentTemplateService.submitReview(params));
    }

    /**
     * 组件模板库-目录
     */
    @ApiOperation("组件模板库-目录")
    @GetMapping("/componentTemplateLibraryCategory")
    public R<List<VisualComponentTemplateCategory>> componentTemplateLibraryCategory(String name) {
        return R.data(visualComponentTemplateCategoryService.queryMcpAllCategory(name));
    }

    /**
     * 组件模板库-模板
     */
    @ApiOperation("组件模板库-模板")
    @GetMapping("/componentTemplateLibrary")
    public R<Page<VisualComponentTemplate>> componentTemplateLibrary(VisualComponentTemplatePageParams params) {
        return R.data(visualComponentTemplateService.getBaseMapper().allReviewedTemplate(new Page(params.getPage(), params.getSize()), params.getName(), params.getCategory()));
    }

    @PostMapping("/submitComponentTemplate")
    @ApiOperation("创建修改组件模板")
    public R<Integer> submitComponentTemplate(@RequestBody SubmitComponentTemplateParams params) {
        Integer l = visualComponentTemplateService.submitComponentTemplate(params);
        if (l==null){
            return R.fail("创建失败");
        }
        return R.data(l);
    }


    @ApiOperation("复制组件模板")
    @PostMapping("/copyComponentTemplate/{id}")
    public R<Boolean> copyComponentTemplate(@PathVariable Long id){
        return R.data(visualComponentTemplateService.copyComponentTemplate(id));
    }


    @ApiOperation("组件模板详情")
    @GetMapping("/details/{id}")
    public R<VisualComponentTemplate> details(@PathVariable Long id){
        VisualComponentTemplate details = visualComponentTemplateService.details(id);
        if (details == null) {
            return R.fail("数据不存在");
        }
        return R.data(details);
    }


    /**
     * 修改组件模板的基本信息
     *
     * @return
     */
    @PostMapping("/updateBasicInfo")
    @ApiOperation("修改组件模板的基本信息")
    public R<Boolean> updateComponentTemplateBasicInfo(@RequestBody UpdateComponentTemplateBasicInfoParams params) {
        return R.data(visualComponentTemplateService.updateComponentTemplateBasicInfo(params));
    }

    @PostMapping("/del/{id}")
    @ApiOperation("删除组件模板")
    public R<Boolean> deleteComponentTemplate(@PathVariable Long id) {
        return R.status(visualComponentTemplateService.deleteComponentTemplate(id));
    }

}
