package org.springblade.visual.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.core.tool.api.R;
import org.springblade.visual.dto.EnableParams;
import org.springblade.visual.dto.PageComponentTemplateParams;
import org.springblade.visual.dto.ReviewComponentTemplateParams;
import org.springblade.visual.entity.VisualComponentTemplate;
import org.springblade.visual.entity.VisualComponentTemplateCategory;
import org.springblade.visual.entity.VisualReviewLog;
import org.springblade.visual.service.VisualComponentTemplateManageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 可视化组件模板管理
 */
@Api(value = "(中台)可视化组件模板管理", tags = "(中台)可视化组件模板管理")
@RestController
@RequestMapping("/visual/component/template/manage")
public class VisualComponentTemplateManageController {

    @Autowired
    private VisualComponentTemplateManageService componentTemplateManageService;


    /**
     * 创建修改目录
     */
    @PostMapping("/submitCategory")
    @ApiOperation("创建 修改目录")
    public R<Boolean> createCategory(@RequestBody VisualComponentTemplateCategory category) {
        return R.data(componentTemplateManageService.submitCategory(category));
    }


    /**
     * 删除分类
     */
    @PostMapping("/editCategory/{id}")
    @ApiOperation("删除分类")
    public R<Boolean> deleteCategory(@PathVariable Long id) {
        return R.data(componentTemplateManageService.deleteCategory(id));
    }

    /**
     * 查询所有目录(租户id是null的数据)
     * 然后通过查询到的数据的parent_id来构建一个树结构
     */
    @ApiOperation("查询所有目录")
    @GetMapping("/queryAllCategory")
    public R<List<VisualComponentTemplateCategory>> queryAllCategory(String name) {
        return R.data(componentTemplateManageService.queryAllCategory(name));
    }

    /**
     * 组件模板列表
     */
    @GetMapping("/page")
    @ApiOperation("组件模板列表")
    public R<IPage<VisualComponentTemplate>> pageComponentTemplate(PageComponentTemplateParams params) {
        return R.data(componentTemplateManageService.pageComponentTemplate(params));
    }

    @ApiOperation("模板审核")
    @PostMapping("/reviewTemplate")
    public R reviewTemplate(@RequestBody ReviewComponentTemplateParams params) {
        return R.status(componentTemplateManageService.reviewTemplate(params));
    }

    @ApiOperation("启用禁用模板")
    @PostMapping("/enableTemplate")
    public R enableTemplate(@RequestBody EnableParams params) {
        return R.status(componentTemplateManageService.enableTemplate(params));
    }

    @ApiOperation("获取审核记录")
    @GetMapping("/listReviewLogs/{id}")
    public R<List<VisualReviewLog>> listReviewLogs(@PathVariable Long id) {
        return R.data(componentTemplateManageService.listReviewLogs(id));
    }


    @ApiOperation("创建人名字集合")
    @GetMapping("/createNames")
    public R createNames() {
        return R.data(componentTemplateManageService.getTemplateCreateUserName());
    }
}
