package org.springblade.visual.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.core.boot.ctrl.BladeController;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.visual.dto.ListTemplateParams;
import org.springblade.visual.dto.SubmitTemplateReviewParams;
import org.springblade.visual.entity.VisualDataTemplate;
import org.springblade.visual.entity.VisualDataTemplateCategory;
import org.springblade.visual.entity.VisualReviewLog;
import org.springblade.visual.mapper.VisualDataTemplateCategoryMapper;
import org.springblade.visual.service.VisualDataTemplateCategoryService;
import org.springblade.visual.service.VisualDataTemplateService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/visual/template")
@Api(tags = "(BI)目录和模板相关接口")
public class VisualDataTemplateController extends BladeController {

    @Autowired
    private VisualDataTemplateService visualDataTemplateService;
    @Autowired
    private VisualDataTemplateCategoryMapper visualDataTemplateCategoryMapper;
    @Autowired
    private VisualDataTemplateCategoryService visualDataTemplateCategoryService;

    @ApiOperation("所有目录")
    @GetMapping("/myCategoryList")
    public R<List<VisualDataTemplateCategory>> categoryList(String name) {
        return R.data(visualDataTemplateService.categoryList(name));
    }

    @ApiOperation("新增/修改目录")
    @PostMapping("/submitCategory")
    public R submitCategory(@RequestBody VisualDataTemplateCategory category) {
        //判重
        List<VisualDataTemplateCategory> list = visualDataTemplateCategoryService.lambdaQuery()
                .eq(VisualDataTemplateCategory::getName, category.getName())
                .eq(VisualDataTemplateCategory::getParentId,category.getParentId())
                .ne(category.getId() != null, VisualDataTemplateCategory::getId, category.getId()).list();
        if (CollectionUtil.isNotEmpty(list)) return R.fail("名字重复");
        return R.status(visualDataTemplateService.submitCategory(category));
    }

    @ApiOperation("删除目录")
    @PostMapping("/deleteCategory")
    public R deleteCategory(Long id) {

        return R.status(visualDataTemplateService.deleteCategory(id));
    }

    @ApiOperation("模板分页")
    @GetMapping("/myTemplates/page")
    public R<Page<VisualDataTemplate>> listTemplates(ListTemplateParams params) {
        params.setCreateUser(SecureUtil.getUserId());
        return R.data(visualDataTemplateService.listTemplates(params));
    }

    /**
     * 修改大屏模板基本信息
     *
     * @return
     */
    @PostMapping("/updateTemplateBasicInfo")
    @ApiOperation("修改大屏模板基本信息")
    public R<Boolean> updateTemplateBasicInfo(@RequestBody VisualDataTemplate template) {
        return R.status(visualDataTemplateService.updateTemplateBasicInfo(template));
    }

    @PostMapping("/removeTemplate/{id}")
    @ApiOperation("删除模板")
    public R<Boolean> removeTemplate(@PathVariable Long id) {
        return R.status(visualDataTemplateService.removeTemplate(id));
    }

    @PostMapping("/copyTemplate/{id}")
    @ApiOperation("复制模板")
    public R<Boolean> copyTemplate(@PathVariable Long id) {
        return R.status(visualDataTemplateService.copyTemplate(id, getUser()));
    }

    @PostMapping("/checkOperationPermissions/{id}")
    @ApiOperation("校验操作权限")
    public R<Boolean> checkOperationPermissions(@PathVariable Long id){
        return R.status(visualDataTemplateService.checkOperationPermissions(id));
    }

    @ApiOperation("获取审核记录")
    @GetMapping("/reviewLogs/{id}")
    public R<List<VisualReviewLog>> listReviewLogs(@PathVariable Long id) {
        return R.data(visualDataTemplateService.listReviewLogs(id));
    }

    @ApiOperation(value = "发起审核")
    @PostMapping("/submitReview")
    public R submitReview(@RequestBody SubmitTemplateReviewParams params) {
        return R.status(visualDataTemplateService.submitReview(params));
    }

    @ApiOperation("所有审核过的模板(平台)")
    @GetMapping("/allReviewedTemplate")
    public R<Page<VisualDataTemplate>> allReviewedTemplate(
            @RequestParam(defaultValue = "1") Integer page,
            @RequestParam(defaultValue = "10") Integer size,
            String name,
            String category) {
        return R.data(visualDataTemplateService.allReviewedTemplate(page, size, name, category));
    }

    /**
     * 所有目录(平台)
     */
    @ApiOperation("所有目录(平台)")
    @GetMapping("/allCategory")
    public R<List<VisualDataTemplateCategory>> allCategory(String name) {
        return R.data(visualDataTemplateService.allMcpCategory(name));
    }

}
