package org.springblade.visual.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springblade.core.tool.api.R;
import org.springblade.visual.dto.EnableParams;
import org.springblade.visual.dto.ListTemplateParams;
import org.springblade.visual.dto.ReviewTemplateParams;
import org.springblade.visual.entity.VisualDataTemplateCategory;
import org.springblade.visual.entity.VisualReviewLog;
import org.springblade.visual.service.VisualDataTemplateManageService;
import org.springblade.visual.service.VisualDataTemplateService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/manage/visual/template")
@Api(tags = "(中台)模板管理")
public class VisualDataTemplateManageController {

    @Autowired
    private VisualDataTemplateManageService visualDataTemplateManageService;
    @Autowired
    private VisualDataTemplateService visualDataTemplateService;

    /**
     * 创建修改目录
     */
    @PostMapping("/submitCategory")
    @ApiOperation("创建 修改目录")
    public R<Boolean> createCategory(@RequestBody VisualDataTemplateCategory category) {
        return R.data(visualDataTemplateService.submitCategory(category));
    }


    /**
     * 删除分类
     */
    @PostMapping("/editCategory/{id}")
    @ApiOperation("删除分类")
    public R<Boolean> deleteCategory(@PathVariable Long id) {
        return R.data(visualDataTemplateManageService.deleteCategory(id));
    }

    /**
     * 查询所有目录(租户id是null的数据)
     */
    @ApiOperation("查询所有目录")
    @GetMapping("/queryAllCategory")
    public R<List<VisualDataTemplateCategory>> queryAllCategory(String name) {
        return R.data(visualDataTemplateService.allMcpCategory(name));
    }

    @ApiOperation("创建人名字集合")
    @GetMapping("/createNames")
    public R createNames() {
        return R.data(visualDataTemplateManageService.getTemplateCreateUserName());
    }

    @ApiOperation("模板分页")
    @GetMapping("/templates/page")
    public R listTemplates(ListTemplateParams params) {
        return R.data(visualDataTemplateManageService.listTemplates(params));
    }

    @ApiOperation("模板审核")
    @PostMapping("/reviewTemplate")
    public R reviewTemplate(@RequestBody ReviewTemplateParams params) {
        return R.status(visualDataTemplateManageService.reviewTemplate(params));
    }

    @ApiOperation("模板修改分类")
    @PostMapping("/updateCategory/{id}/{category}")
    public R<Boolean> updateCategory(@PathVariable("id") Long id, @PathVariable("category") Long category) {
        return R.status(visualDataTemplateManageService.updateCategory(id, category));
    }

    @ApiOperation("启用禁用模板")
    @PostMapping("/enableTemplate")
    public R enableTemplate(@RequestBody EnableParams params) {
        return R.status(visualDataTemplateManageService.enableTemplate(params));
    }

    @ApiOperation("获取审核记录")
    @GetMapping("/listReviewLogs/{id}")
    public R<List<VisualReviewLog>> listReviewLogs(@PathVariable Long id) {
        return R.data(visualDataTemplateManageService.listReviewLogs(id));
    }

}
