/*
 * Copyright (c) 2018-2028, aokunsang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.visual.enums;

import java.math.BigDecimal;
import java.math.RoundingMode;

/**
 * 同环比，当compareDataType={@link CompareDataTypeEnum#PERCENT}时，且差值百分比为负数时，
 *   提供3种计算方式
 *
 * @author wookvn
 * @datetime 2024-10-15 17:51:36
 */
public enum CompareRatioCalcTypeEnum {

    /**
     * (1-本期数据/上期数据) * 100%
     */
    NORMAL{
        @Override
        public BigDecimal apply(BigDecimal curData, BigDecimal preData) {
            return BigDecimal.ONE.subtract(curData.divide(preData, CompareDataTypeEnum.RADIX_POINT, RoundingMode.HALF_UP));
        }
    },

    /**
     * (本期数据/上期数据-1) * 100%
     */
    ABSOLUTE{
        @Override
        public BigDecimal apply(BigDecimal curData, BigDecimal preData) {
            return curData.divide(preData, CompareDataTypeEnum.RADIX_POINT, RoundingMode.HALF_UP).subtract(BigDecimal.ONE);
        }
    },

    /**
     * 展示为-
     */
    UNKNOWN{
        @Override
        public BigDecimal apply(BigDecimal curData, BigDecimal preData) {
            return null;
        }
    },
    ;

    /**
     * <p>计算方式</p>
     * @author wookvn
     * @datetime 2024-10-16 17:32:01
     * @param curData: 上期数据
     * @param preData: 下期数据
     * @return java.math.BigDecimal
     **/
    public abstract BigDecimal apply(BigDecimal curData, BigDecimal preData);
}
