/*
 * Copyright (c) 2018-2028, aokunsang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.visual.enums;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.Getter;
import org.springblade.datasource.enums.FunctionMethodEnum;

import java.util.Arrays;

/**
 * 对比类型
 *
 * @author wookvn
 * @datetime 2024-10-15 17:50:47
 */
@Getter
@AllArgsConstructor
public enum CompareTypeEnum {

    /**
     * 年环比
     */
    YEAR_CYCLE("年环比", new FunctionMethodEnum[]{FunctionMethodEnum.EXTRACT_YEAR}),

    /**
     * 年同比
     */
    YEAR_SELF("年同比", new FunctionMethodEnum[]{FunctionMethodEnum.EXTRACT_QUARTER, FunctionMethodEnum.EXTRACT_MONTH, FunctionMethodEnum.EXTRACT_WEEK, FunctionMethodEnum.EXTRACT_DAY}),

    /**
     * 季环比
     */
    QUARTER_CYCLE("季环比", new FunctionMethodEnum[]{FunctionMethodEnum.EXTRACT_QUARTER}),

    /**
     * 月环比
     */
    MONTH_CYCLE("月环比", new FunctionMethodEnum[]{FunctionMethodEnum.EXTRACT_MONTH}),

    /**
     * 月同比
     */
    MONTH_SELF("月同比", new FunctionMethodEnum[]{FunctionMethodEnum.EXTRACT_DAY}),

    /**
     * 周环比
     */
    WEEK_CYCLE("周环比", new FunctionMethodEnum[]{FunctionMethodEnum.EXTRACT_WEEK}),

    /**
     * 周同比
     */
    WEEK_SELF("周同比", new FunctionMethodEnum[]{FunctionMethodEnum.EXTRACT_DAY}),

    /**
     * 日环比
     */
    DAY_CYCLE("日环比", new FunctionMethodEnum[]{FunctionMethodEnum.EXTRACT_DAY}),

    ;

    private String message;
    private FunctionMethodEnum[] functionMethods;

    /**
     * <p>校验是否支持该方法</p>
     * @author wookvn
     * @datetime 2024-10-16 11:29:33
     * @param functionMethod: 日期提取方法
     * @return boolean
     **/
    public boolean supports(FunctionMethodEnum functionMethod) {
        return Arrays.stream(this.functionMethods).anyMatch(item -> item == functionMethod);
    }

}