package org.springblade.visual.mapper;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;
import org.apache.ibatis.annotations.Update;
import org.mapstruct.Mapper;
import org.springblade.visual.entity.VisualComponentTemplate;

import java.util.List;

@Mapper
public interface VisualComponentTemplateMapper extends BaseMapper<VisualComponentTemplate> {

    /**
     * 查询平台模版库
     * 不需要租户条件
     */
    @InterceptorIgnore(tenantLine = "true")
    @Select("<script>" +
            "select * FROM visual_component_template " +
            "WHERE status = 1  " +
            "<if test='name != null  and name != \"\"'> AND name like CONCAT('%', #{name}, '%') </if>" +
            "<if test='category != null'> AND mcp_category = #{category} </if>" +
            "ORDER BY create_time desc" +
            "</script>")
    Page<VisualComponentTemplate> allReviewedTemplate(Page page, @Param("name") String name, @Param("category") Long category);


    @InterceptorIgnore(tenantLine = "true")
    @Update("update visual_component_template set use_times = use_times + 1 where id = #{id};")
    void addUseTimes(@Param("id") Long id);

    @InterceptorIgnore(tenantLine = "true")
    @Update({
            "<script>",
            "UPDATE visual_component_template ",
            "SET use_times = use_times + 1 ",
            "WHERE id IN ",
            "<foreach item='item' index='index' collection='idList' open='(' separator=',' close=')'>",
            "#{item}",
            "</foreach>",
            "</script>"
    })
    void addUseTimesBatch(@Param("idList") List<Long> idList);
}
