/**
 *
 */
package org.springblade.visual.service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

import cn.hutool.core.util.IdUtil;
import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.cube.enums.DimensionTypeEnum;
import org.springblade.cube.enums.ParamTypeEnum;
import org.springblade.cube.enums.WeiduTypeEnum;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.model.CubeGlobalParam;
import org.springblade.cube.service.CubeDataPermissionService;
import org.springblade.cube.service.CubeDataService;
import org.springblade.cube.service.CubeGlobalParamService;
import org.springblade.cube.utils.CubeConstants;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.datasource.context.DialectContext;
import org.springblade.datasource.enums.DateFormatEnum;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.enums.FunctionMethodEnum;
import org.springblade.datasource.enums.NumberFormatEnum;
import org.springblade.datasource.enums.ViewColumnTypeEnum;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.report.service.IReportDatasourceService;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.ComponentFilter;
import org.springblade.visual.dto.ComponentFilterCondition;
import org.springblade.visual.dto.ComponentFilterDto;
import org.springblade.visual.dto.ComponentFilterItem;
import org.springblade.visual.dto.ComponentFilterPlus;
import org.springblade.visual.dto.ComponentFilterRelation;
import org.springblade.visual.dto.ComponentSort;
import org.springblade.visual.dto.CubePermParseDto;
import org.springblade.visual.dto.PreviewBaseDto;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.dto.WarningFilterRelativeTime;
import org.springblade.visual.dto.WarningFilterTime;
import org.springblade.visual.enums.CommonFunctionEnum;
import org.springblade.visual.enums.ComponentTypeEnum;
import org.springblade.visual.enums.FilterConnectTimeEnum;
import org.springblade.visual.enums.FilterConnectTypeEnum;
import org.springblade.visual.enums.FilterTimeCalcEnum;
import org.springblade.visual.service.component.VisualComponentCompareService;
import org.springblade.visual.utils.DateSqlUtils;
import org.springblade.visual.utils.Tuple2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Tuple;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 */
@Slf4j
@Service
public class QuerySqlService {

    @Autowired
    private CubeDataService cubeDataService;

    @Autowired
    private IReportDatasourceService reportDatasourceService;

    @Autowired
    private CubeDataPermissionService cubeDataPermissionService;

    @Autowired
    private DialectContext dialectContext;

    @Autowired
    private CubeGlobalParamService cubeGlobalParamService;

    @Autowired
    private VisualComponentCompareService visualComponentCompareService;

    /**
     * 常规组件 - 解析查询SQL
     *
     * @param cubeDataId
     * @param previewDto
     * @return
     */
    public R<QuerySqlParam> parseQuerySql(Integer cubeDataId, PreviewBaseDto previewDto, BladeUser bladeUser) {
        return parseQuerySql(cubeDataId, previewDto, bladeUser, ComponentTypeEnum.GENERAL);
    }

    /**
     * 解析查询SQL
     *
     * @param cubeDataId
     * @param previewDto
     * @return
     */
    public R<QuerySqlParam> parseQuerySql(Integer cubeDataId, PreviewBaseDto previewDto, BladeUser bladeUser, ComponentTypeEnum compType) {
        CubeData cubeData = cubeDataService.getById(cubeDataId);
        if (cubeData == null) {
            log.error("数据集不存在|id={}", cubeDataId);
            return R.fail("数据集不存在");
        }
        //2.查询数据源
        ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
        if (reportDatasource == null) {
            log.error("数据源不存在|id={}", cubeData.getDatasourceId());
            return R.fail("数据源不存在");
        }
        return parseQuerySql(cubeData, reportDatasource, previewDto, bladeUser, compType);
    }

    /**
     * 解析查询SQL
     *
     * @param cubeData
     * @param reportDatasource
     * @param previewDto
     * @param bladeUser
     * @param compType
     * @return
     */
    public R<QuerySqlParam> parseQuerySql(CubeData cubeData, ReportDatasource reportDatasource, PreviewBaseDto previewDto, BladeUser bladeUser, ComponentTypeEnum compType) {
        //构造全局变量
        Map<String, List<Object>> extraParams = buildGlobalParams(previewDto.getExtraParams(), bladeUser);
        DsTypeEnum dsTypeEnum = reportDatasource.getDsTypeSyl();

        String fromSql = cubeData.getQuerySql();
        //2.构造Select模块
        List<ComponentColumn> columnList = JSON.parseArray(previewDto.getColumnList(), ComponentColumn.class);
        //补全日期查询的时间列名
        ComponentColumn filterColum = null;
        //对指标维度字段做初始化操作，并且做异常校验
        for (int i = 0; i < columnList.size(); i++) {
            ComponentColumn item = columnList.get(i);
            //设置uniqId保证每列的唯一性【注意：不能使用cubeColumnId作为唯一值处理，存在日期自动拆分业务】
            if(StrUtil.isBlank(item.getUniqId())) {
                item.setUniqId(IdUtil.getSnowflakeNextIdStr());
            }
            //设置别名
            if (StrUtil.isBlank(item.getColumnAliasName())) {
                String columnAliasName = CubeConstants.COLUMN_ALIAS_NAME_PREFIX + i;
                item.setColumnAliasName(dsTypeEnum.isFieldLowerCase() ? columnAliasName.toLowerCase() : columnAliasName);
            }
            //设补全日期的列
            if(previewDto.getIsDateComplete() && item.getColumnName().equals(previewDto.getFilterColum())) {
                filterColum = item;
            }
            //计算字段SQL替换全局变量
            item.setColumnSql(CubeUtils.formatVirtualSqlFragment(item.getColumnSql(), extraParams));
        }

        //同环比的前置处理
        visualComponentCompareService.preCompareHandle(columnList, compType, reportDatasource.getDsTypeSyl());

        //存储select和group的sql片段
        List<String> selectSqlList = new ArrayList<String>();
        List<String> groupBySqlList = new ArrayList<String>();

        //循环遍历维度指标字段列表
        for (ComponentColumn column : columnList) {
            String columnName = column.getColumnSql();
            Integer virtualAggregationStatus = Optional.<Integer>ofNullable(column.getVirtualAggregationStatus()).orElse(0);
            //如果清单组件,并且计算字段包含聚合函数，抛出异常提示
            if (compType == ComponentTypeEnum.DETAIL && virtualAggregationStatus == 1) {
                return R.fail("清单组件不能包含聚合函数，请检查数据集字段【" + column.getColumnTitle() + "】");
            }
            //聚合函数[且计算字段是不包含聚合函数的]
            if (column.getAggregationType() != null && column.getDimensionType() == DimensionTypeEnum.ZHIBIAO
                    && compType != ComponentTypeEnum.DETAIL && virtualAggregationStatus == 0) {
                columnName = String.format(column.getAggregationType().getMethod(), columnName);
            }
            // 格式化日期
            if (column.getViewDataType() == ViewColumnTypeEnum.DATETIME) {
                //细分日期颗粒度
                if(column.getFunctionMethod() != null) {
                    columnName = dialectContext.getDialect(dsTypeEnum).formatMethodFunction(column.getFunctionMethod(), columnName, columnName);
                //日期yyyy_MM_ddhh$mm$ss不需要格式化字符串，springboot框架默认对date类型json反序列化格式 [主要兼容老版本]
                }else if(column.getTimeDataFormat() != null && column.getTimeDataFormat() != DateFormatEnum.yyyy_MM_ddhh$mm$ss) {
                    columnName = dialectContext.getDialect(dsTypeEnum).formatDateTime(column.getTimeDataFormat(), columnName);
                }
            }
            selectSqlList.add(columnName + " as \"" + column.getColumnAliasName() + "\"");
            //如果是维度，添加到groupBy
            if (column.getDimensionType() == DimensionTypeEnum.WEIDU && compType != ComponentTypeEnum.DETAIL) {
                //ORACLE的groupBy不能使用别名，clickhouse和mysql可行
                if(dsTypeEnum == DsTypeEnum.clickhouse || dsTypeEnum == DsTypeEnum.mysql) {
                    groupBySqlList.add(column.getColumnAliasName());
                } else {
                    groupBySqlList.add(columnName);
                }
            }
        }
        StringBuilder strBuilder = new StringBuilder("select ").append(String.join(",", selectSqlList)).append(" from ").append(fromSql);
        //解析where和having子句
        Tuple whereHavingSql = parseWhereHavingSql(cubeData, dsTypeEnum, previewDto, bladeUser, extraParams);
        String whereSql = whereHavingSql.get(0), havingSql = whereHavingSql.get(1);
        if (StrUtil.isNotBlank(whereSql)) {
            strBuilder.append(" where " + whereSql);
        }
        //清单组件不需要groupBy和having
        if (compType != ComponentTypeEnum.DETAIL) {
            //groupby子句
            if (!groupBySqlList.isEmpty()) {
                strBuilder.append(" group by ").append(String.join(",", groupBySqlList));
            }
            //having子句
            if (StrUtil.isNotBlank(havingSql)) {
                strBuilder.append(" having " + havingSql);
            }
        }

        Map<String, ComponentColumn> columnMap = columnList.stream().collect(Collectors.toMap(ComponentColumn::getUniqId, t -> t, (v1, v2) -> v1));
        //兼容处理
        Map<Integer, ComponentColumn> columnIdMap = columnList.stream().collect(Collectors.toMap(ComponentColumn::getCubeColumnId, t -> t, (v1, v2) -> v1));

        //构造排序条件
        List<ComponentSort> sortList = Optional.ofNullable(JSON.parseArray(previewDto.getSortList(), ComponentSort.class)).orElse(new ArrayList<>());
        List<String> sortSqlList = new ArrayList<String>();
        for (ComponentSort sort : sortList) {
            ComponentColumn column = sort.getColumn();
            ComponentColumn componentColumn = columnMap.getOrDefault(column.getUniqId(), columnIdMap.get(column.getCubeColumnId()));
            //如果别名不在select中，不能放到orderBy中
            if (componentColumn != null) {
                sortSqlList.add(componentColumn.getColumnAliasName() + " " + sort.getOrderType().name());
            }
        }
        if (!sortSqlList.isEmpty()) {
            strBuilder.append(" order by ").append(String.join(",", sortSqlList));
        }
        if (CubeUtils.checkSqlInject(strBuilder.toString())) {
            log.info("sqlInject:warning|sql=" + strBuilder.toString());
            return R.fail("有SQL注入风险，请检查");
        }

        String sql = strBuilder.toString();
        Map<String, Object> map = null;
        if (filterColum != null) {
            map = DateSqlUtils.getSql(filterColum, previewDto.getSelectTime(), strBuilder.toString(), columnList, reportDatasource);
            sql = map.get("sql").toString();
        }
        QuerySqlParam querySqlParam = new QuerySqlParam();
        querySqlParam.setQuerySql(sql);
        querySqlParam.setCubeData(cubeData);
        querySqlParam.setReportDatasource(reportDatasource);
        querySqlParam.setSelectColumnList(columnList);
        querySqlParam.setExcludeColumnIdList(whereHavingSql.get(2));
        if (CollectionUtil.isNotEmpty(map)) {
            List<String> selectTime = map.containsKey("dataTime") ? (List<String>) map.get("dataTime") : null;
            List<String> hourTime = map.containsKey("hourTime") ? (List<String>) map.get("hourTime") : null;
            String timeColum = map.containsKey("timeColum") ? map.get("timeColum").toString() : null;
            querySqlParam.setSelectTime(selectTime);
            querySqlParam.setHourTime(hourTime);
            querySqlParam.setTimeColum(timeColum);
        }
        return R.data(querySqlParam);
    }

    /**
     * <p>逻辑分页的sql解析</p>
     * @author wookvn
     * @datetime 2024-10-14 11:11:40
     * @param cubeDataId: 数据集ID
     * @param previewDto: 核心参数对象
     * @param bladeUser: 当前登录用户
     * @return org.springblade.core.tool.api.R<org.springblade.visual.dto.QuerySqlParam>
     **/
    public R<QuerySqlParam> parseLogicPageQuerySql(Integer cubeDataId, PreviewBaseDto previewDto, BladeUser bladeUser) {
        CubeData cubeData = cubeDataService.getById(cubeDataId);
        if (cubeData == null) {
            log.error("数据集不存在|id={}", cubeDataId);
            return R.fail("数据集不存在");
        }
        //2.查询数据源
        ReportDatasource reportDatasource = reportDatasourceService.getById(cubeData.getDatasourceId());
        if (reportDatasource == null) {
            log.error("数据源不存在|id={}", cubeData.getDatasourceId());
            return R.fail("数据源不存在");
        }
        //遍历维度
        Map<String, List<Object>> extraParams = buildGlobalParams(previewDto.getExtraParams(), bladeUser);
        DsTypeEnum dsTypeEnum = reportDatasource.getDsTypeSyl();

        //2.构造Select模块
        List<ComponentColumn> columnList = JSON.parseArray(previewDto.getColumnList(), ComponentColumn.class);
        //初始化别名
        for (int i = 0; i < columnList.size(); i++) {
            ComponentColumn item = columnList.get(i);
            item.setColumnAliasName(CubeConstants.COLUMN_ALIAS_NAME_PREFIX + i);
        }
        List<String> weiduColumnList = new ArrayList<>();
        List<String> groupList = new ArrayList<>(), orderList = new ArrayList<>();

        for (ComponentColumn column : columnList) {
            //行维度做展示操作
            if (column.getDimensionType() == DimensionTypeEnum.WEIDU && column.getWeiduType() == WeiduTypeEnum.ROW) {
                String columnName = column.getColumnSql();
                //日期需要格式化
                if (column.getViewDataType() == ViewColumnTypeEnum.DATETIME && (column.getTimeDataFormat() != null || column.getFunctionMethod() != null)) {
                    //细分日期颗粒度
                    if(column.getFunctionMethod() != null) {
                        columnName = dialectContext.getDialect(dsTypeEnum).formatMethodFunction(column.getFunctionMethod(), columnName, columnName);
                        //日期yyyy_MM_ddhh$mm$ss不需要格式化字符串，springboot框架默认对date类型json反序列化格式 [主要兼容老版本]
                    }else if(column.getTimeDataFormat() != null) {
                        columnName = dialectContext.getDialect(dsTypeEnum).formatDateTime(column.getTimeDataFormat(), columnName);
                    }
                }
                weiduColumnList.add(columnName + " as \"" + column.getColumnAliasName() + "\"");
                groupList.add(columnName);
                orderList.add(column.getColumnAliasName());
            }
        }

        Tuple whereHavingSql = parseWhereHavingSql(cubeData, dsTypeEnum, previewDto, bladeUser, extraParams);
        String whereSql = whereHavingSql.get(0);

        String querySql = String.format("select %s from %s%s group by %s order by %s",
                String.join(",", weiduColumnList),
                cubeData.getQuerySql(),
                StrUtil.isNotBlank(whereSql) ? (" where " + whereSql) : "",
                String.join(",", groupList),
                String.join(",", orderList));
        QuerySqlParam querySqlParam = new QuerySqlParam();
        querySqlParam.setQuerySql(querySql);
        querySqlParam.setCubeData(cubeData);
        querySqlParam.setReportDatasource(reportDatasource);
        querySqlParam.setSelectColumnList(columnList);
        querySqlParam.setExcludeColumnIdList(whereHavingSql.get(2));
        return R.data(querySqlParam);
    }

    /**
     * 解析使用到的字段ID列表
     *
     * @param previewDto
     * @return
     */
    public Set<Integer> parseUsedColumnIdList(PreviewBaseDto previewDto) {
        //字段ID列表
        Set<Integer> columnIdList = new HashSet<Integer>();
        //字段
        Optional.ofNullable(JSON.parseArray(previewDto.getColumnList(), ComponentColumn.class)).orElse(new ArrayList<>()).stream().map(ComponentColumn::getCubeColumnId).forEach(columnIdList::add);
        //过滤器字段
        Optional.ofNullable(JSON.parseArray(previewDto.getFilterList(), ComponentFilter.class)).orElse(new ArrayList<>()).stream().map(item -> item.getColumn().getCubeColumnId()).forEach(columnIdList::add);
        //排序字段
        Optional.ofNullable(JSON.parseArray(previewDto.getSortList(), ComponentSort.class)).orElse(new ArrayList<>()).stream().map(item -> item.getColumn().getCubeColumnId()).forEach(columnIdList::add);
        return columnIdList;
    }

    /**
     * 解析相对时间和标准时间
     *
     * @param filterTimeJson
     * @return
     */
    public List<ComponentFilter> parseFilterTime(String filterTimeJson, DsTypeEnum dsTypeEnum) {
        if (StrUtil.isEmpty(filterTimeJson)) {
            return null;
        }
        WarningFilterTime filterTime = JSON.parseObject(filterTimeJson, WarningFilterTime.class);
        //时间字段和相对时间对象 不能为空
        if (filterTime.getTimeColumn() == null || filterTime.getRelativeTime() == null) {
            return null;
        }
        if (filterTime.getCalcType() == FilterTimeCalcEnum.SPECIAL &&
                filterTime.getTimeColumn().getViewDataType() != ViewColumnTypeEnum.NUMBER) {
            log.warn("timeColumn is not a Number column!!");
            return null;
        }
        if (filterTime.getCalcType() == FilterTimeCalcEnum.NORMAL &&
                filterTime.getTimeColumn().getViewDataType() != ViewColumnTypeEnum.DATETIME) {
            log.warn("timeColumn is not a DateTime column!!");
            return null;
        }
        //计算出2个时间[开始时间和结束时间]
        WarningFilterRelativeTime relativeTime = filterTime.getRelativeTime();
        LocalDateTime startDateTime = relativeTime.getLeftModifier().getTime(relativeTime.getLeftVal(), filterTime.getTimeType());
        LocalDateTime endDateTime = relativeTime.getRightModifier().getTime(relativeTime.getRightVal(), filterTime.getTimeType());
        List<Object> values = new ArrayList<Object>();
        if (filterTime.getCalcType() == FilterTimeCalcEnum.NORMAL) {
            ComponentColumn timeColumn = filterTime.getTimeColumn();
            values.add(DateUtil.format(startDateTime, timeColumn.getTimeDataFormat().getFormat()));
            values.add(DateUtil.format(endDateTime, timeColumn.getTimeDataFormat().getFormat()));
        } else {
            values.add(filterTime.getTimeType().formatInteger(startDateTime));
            values.add(filterTime.getTimeType().formatInteger(endDateTime));
        }
        List<ComponentFilter> filterList = new ArrayList<ComponentFilter>();
        //构造时间字段过滤器
        ComponentFilter timeColumnFilter = new ComponentFilter(filterTime.getTimeColumn());
        timeColumnFilter.setConditions(Arrays.asList(new ComponentFilterCondition(CommonFunctionEnum.between, values)));
        filterList.add(timeColumnFilter);
        //如果时间计算方式为特殊时间，需要添加时间纬度过滤器
        if (filterTime.getCalcType() == FilterTimeCalcEnum.SPECIAL && filterTime.getTimeTypeColumn() != null) {
            ComponentFilter timeTypeColumnFilter = new ComponentFilter(filterTime.getTimeTypeColumn());
            timeTypeColumnFilter.setConditions(ListUtil.toList(new ComponentFilterCondition(CommonFunctionEnum.eq, Arrays.asList(filterTime.getTimeType().name()))));
            filterList.add(timeTypeColumnFilter);
        }
        return filterList;
    }

    /**
     * 解析where和having子句
     *
     * @param cubeData
     * @return
     */
    public Tuple parseWhereHavingSql(CubeData cubeData, DsTypeEnum dsTypeEnum, PreviewBaseDto previewDto, BladeUser bladeUser, Map<String, List<Object>> extraParams) {
        List<String> whereSqlList = new ArrayList<>();
        //解析数据集全局过滤器
        if (StrUtil.isNotBlank(cubeData.getFilterJson())) {
            PreviewBaseDto cubePreviewDto = JSON.parseObject(cubeData.getFilterJson(), PreviewBaseDto.class);
            //解析SQL
            Tuple2<String, String> whereHavingSqlTuple = parseFilters(JSON.parseArray(cubePreviewDto.getFilterList(), ComponentFilter.class),
                    JSON.parseObject(cubePreviewDto.getFilterRelation(), ComponentFilterRelation.class), dsTypeEnum, extraParams);
            if (StrUtil.isNotBlank(whereHavingSqlTuple.getT1())) {
                whereSqlList.add(whereHavingSqlTuple.getT1());
            }
        }
        //解析当前登录用户的行/列权限
        CubePermParseDto cubePermParseDto = cubeDataPermissionService.parseCubePermission(cubeData.getId(), bladeUser, extraParams);
        if (StrUtil.isNotBlank(cubePermParseDto.getFilterSql())) {
            whereSqlList.add(cubePermParseDto.getFilterSql());
        }
        //构建过滤条件
        List<ComponentFilter> filterList = Optional.ofNullable(JSON.parseArray(previewDto.getFilterList(), ComponentFilter.class)).orElse(new ArrayList<ComponentFilter>());
        //执行前置操作，是否有时间JSON，进行查询操作
        List<ComponentFilter> filterTimeList = parseFilterTime(previewDto.getFilterTimeJson(), dsTypeEnum);
        if (filterTimeList != null) {
            filterList.addAll(filterTimeList);
        }
        //构造冗余过滤条件
        List<ComponentFilter> extFilterList = Optional.ofNullable(JSON.parseArray(previewDto.getExtFilterJson(), ComponentFilter.class)).orElse(new ArrayList<ComponentFilter>());
        if (CollectionUtil.isNotEmpty(extFilterList)) {
            filterList.addAll(extFilterList);
        }
        //解析过滤器以及过滤器关系
        ComponentFilterRelation componentFilterRelation = JSON.parseObject(previewDto.getFilterRelation(), ComponentFilterRelation.class);
        Tuple2<String, String> whereHavingSqlTuple = parseFilters(filterList, componentFilterRelation, dsTypeEnum, extraParams);
        String whereSql = whereHavingSqlTuple.getT1(), havingSql = whereHavingSqlTuple.getT2();
        //where子句
        if (StrUtil.isNotBlank(whereSql)) {
            whereSqlList.add(whereSql);
        }
        return new Tuple(String.join(" AND ", whereSqlList), havingSql, cubePermParseDto.getExcludeColumnIdList());
    }

    /**
     * <p>解析过滤器</p>
     */
    public Tuple2<String, String> parseFilters(List<ComponentFilter> filterList, ComponentFilterRelation componentFilterRelation, DsTypeEnum dsTypeEnum, Map<String, List<Object>> extraParams) {
        return parseFilters(filterList, componentFilterRelation, dsTypeEnum, extraParams, true);
    }

    /**
     * <p>解析过滤器</p>
     * @author wookvn
     * @datetime 2024-10-23 16:17:11
     * @param filterList: 过滤器列表
     * @param componentFilterRelation: 过滤器关系对象
     * @param dsTypeEnum: 数据源类型
     * @param extraParams: 全局参数值Map
     * @param isReplaceGlobalParams: 是否替换全局参数：当只需要拿到whereSql片段存储到数据库时，不需要解析全局变量；在执行过程中解析【场景：权限组过滤器使用】
     * @return org.springblade.visual.utils.Tuple2<java.lang.String,java.lang.String>
     **/
    public Tuple2<String, String> parseFilters(List<ComponentFilter> filterList, ComponentFilterRelation componentFilterRelation, DsTypeEnum dsTypeEnum, Map<String, List<Object>> extraParams, boolean isReplaceGlobalParams) {
        String whereSql = "", havingSql = "";
        if (CollectionUtil.isNotEmpty(filterList)) {
            //当外部调用该解析器时，判断extraParams是否为null
            extraParams = extraParams != null ? extraParams : CubeConstants.mergeSysUserParams(AuthUtil.getUser(), new HashMap<>());
            //过滤关系
            if (componentFilterRelation != null) {
                //filterRelation包含where和having的过滤关系，依次处理
                Tuple2<String, String> whereSqlTuple = _buildFilters(componentFilterRelation.getWhere(), filterList, dsTypeEnum, extraParams, isReplaceGlobalParams);
                if (whereSqlTuple != null) {
                    whereSql = whereSqlTuple.getT1();
                }
                //having处理
                Tuple2<String, String> havingSqlTuple = _buildFilters(componentFilterRelation.getHaving(), filterList, dsTypeEnum, extraParams, isReplaceGlobalParams);
                if (havingSqlTuple != null) {
                    havingSql = havingSqlTuple.getT2();
                }
                //构造非uniqId的条件集合【主要是全局筛选器，因此只需要处理where】
                List<ComponentFilter> dataList = filterList.stream().filter(f -> StrUtil.isEmpty(f.getUniqId())).collect(Collectors.toList());
                if (!dataList.isEmpty()) {
                    Tuple2<String, String> globalWhereSqlTuple = _buildConditions(dataList, FilterConnectTypeEnum.AND, dsTypeEnum, extraParams, isReplaceGlobalParams);
                    if (globalWhereSqlTuple != null && !"".equals(globalWhereSqlTuple.getT1())) {
                        whereSql += (StrUtil.isNotBlank(whereSql) ? " AND " : "") + globalWhereSqlTuple.getT1();
                    }
                }
            } else { //没有过滤关系
                Tuple2<String, String> whereHavingSqlTuple = _buildConditions(filterList, FilterConnectTypeEnum.AND, dsTypeEnum, extraParams, isReplaceGlobalParams);
                if (whereHavingSqlTuple != null) {
                    whereSql = whereHavingSqlTuple.getT1();
                    havingSql = whereHavingSqlTuple.getT2();
                }
            }
        }
        return new Tuple2<String, String>(whereSql, havingSql);
    }

    /**
     * 构造全局变量参数
     *
     * @param extraParams
     * @return
     */
    public Map<String, List<Object>> buildGlobalParams(Map<String, List<Object>> extraParams, BladeUser bladeUser) {
        Map<String, List<Object>> _extraParams = new HashMap<>();
        //兼容处理：智能预警没有bladeUser情况
        if (bladeUser != null) {
            //添加tenantId查询条件，因为服务化接口会调用该方法
//            List<CubeGlobalParam> gloablParamList = cubeGlobalParamService.list(Wrappers.<CubeGlobalParam>lambdaQuery()
//                    .eq(CubeGlobalParam::getParamType, ParamTypeEnum.BIZ).eq(CubeGlobalParam::getTenantId, bladeUser.getTenantId()));
            List<CubeGlobalParam> gloablParamList = cubeGlobalParamService.listByTypeAndTenantIdFromCache(ParamTypeEnum.BIZ, bladeUser.getTenantId());
                    extraParams = Optional.ofNullable(extraParams).orElse(new HashMap<>());
            if (CollectionUtil.isNotEmpty(gloablParamList)) {
                Map<String, String> _mapData = gloablParamList.stream().collect(Collectors.toMap(CubeGlobalParam::getParamKey, t -> Optional.ofNullable(t.getDefaultValue()).orElse(""), (a, b) -> b));
                for (Map.Entry<String, String> entry : _mapData.entrySet()) {
                    List<Object> paramValue = extraParams.get(entry.getKey());
                    if (StrUtil.isNotBlank(entry.getValue()) || CollectionUtil.isNotEmpty(paramValue)) {
                        //优先使用paramValue的值
                        _extraParams.put(entry.getKey(), CollectionUtil.isNotEmpty(paramValue) ? paramValue : ListUtil.toList(entry.getValue()));
                    }
                }
            }
        }
        return CubeConstants.mergeAllSysParams(bladeUser, DateFormatEnum.yyyy_MM_ddhh$mm$ss, _extraParams);
    }

    /**
     * 构建Where子句和Having子句
     *
     * @param compFilterDto
     */
    private Tuple2<String, String> _buildFilters(ComponentFilterDto compFilterDto, List<ComponentFilter> filterList, DsTypeEnum dsTypeEnum, Map<String, List<Object>> extraParams, boolean isReplaceGlobalParams) {
        if (compFilterDto == null || compFilterDto.getRelation() == null
                || (CollectionUtil.isEmpty(compFilterDto.getConditions()) && CollectionUtil.isEmpty(compFilterDto.getFilters()))) {
            return null;
        }
        List<String> havingSqlList = new ArrayList<String>();
        //1.构造单个条件集合
        List<String> whereSqlList = new ArrayList<String>();
        if (compFilterDto.getConditions() != null && !compFilterDto.getConditions().isEmpty()) {
            List<String> uniqIdList = compFilterDto.getConditions().stream().filter(f -> StrUtil.isNotBlank(f.getUniqId())).map(ComponentFilterItem::getUniqId).collect(Collectors.toList());
            List<ComponentFilter> filterDataList = filterList.stream().filter(f -> uniqIdList.contains(f.getUniqId())).collect(Collectors.toList());
            Tuple2<String, String> tuple = _buildConditions(filterDataList, compFilterDto.getRelation(), dsTypeEnum, extraParams, isReplaceGlobalParams);
            if (StrUtil.isNotBlank(tuple.getT1())) {
                whereSqlList.add(tuple.getT1());
            }
            if (StrUtil.isNotBlank(tuple.getT2())) {
                havingSqlList.add(tuple.getT2());
            }
        }
        //2.构造多个条件集合
        if (CollectionUtil.isNotEmpty(compFilterDto.getFilters())) {
            for (ComponentFilterPlus filterPlus : compFilterDto.getFilters()) {
                List<String> uniqIdList = filterPlus.getConditions().stream().filter(f -> StrUtil.isNotBlank(f.getUniqId())).map(ComponentFilterItem::getUniqId).collect(Collectors.toList());
                List<ComponentFilter> conds = filterList.stream().filter(f -> uniqIdList.contains(f.getUniqId())).collect(Collectors.toList());
                FilterConnectTypeEnum relation = filterPlus.getRelation();
                if (conds != null && relation != null && !conds.isEmpty()) {
                    Tuple2<String, String> tuple = _buildConditions(conds, relation, dsTypeEnum, extraParams, isReplaceGlobalParams);
                    if (StrUtil.isNotBlank(tuple.getT1())) {
                        whereSqlList.add(tuple.getT1());
                    }
                    if (StrUtil.isNotBlank(tuple.getT2())) {
                        havingSqlList.add(tuple.getT2());
                    }
                }
            }
        }
        return new Tuple2<String, String>(_stringListJoin(whereSqlList, compFilterDto.getRelation()), _stringListJoin(havingSqlList, compFilterDto.getRelation()));
    }

    /**
     * 构造查询条件
     *
     * @param compFilterList
     * @param relation
     * @param dsTypeEnum
     * @return
     */
    private Tuple2<String, String> _buildConditions(List<ComponentFilter> compFilterList, FilterConnectTypeEnum relation, DsTypeEnum dsTypeEnum, Map<String, List<Object>> extraParams, boolean isReplaceGlobalParams) {
        List<String> whereSqlList = new ArrayList<String>();
        List<String> havingSqlList = new ArrayList<String>();
        for (ComponentFilter columnFilter : compFilterList) {
            ComponentColumn column = columnFilter.getColumn();
            String columnName = CubeUtils.formatVirtualSqlFragment(column.getColumnSql(), extraParams);
            //计算过滤时机
            FilterConnectTimeEnum connectTime = columnFilter.getConnectTime();
            //HAVING需要把聚合函数参与进来[如果计算字段已存在聚合函数，不再添加]
            if (connectTime == FilterConnectTimeEnum.HAVING && column.getAggregationType() != null
                    && Optional.<Integer>ofNullable(column.getVirtualAggregationStatus()).orElse(0) == 0) {
                columnName = String.format(column.getAggregationType().getMethod(), columnName);
            }
            //格式化日期
            if (column.getViewDataType() == ViewColumnTypeEnum.DATETIME) {
                //细分日期颗粒度，维度查询
                if(column.getFunctionMethod() != null) {
                    columnName = dialectContext.getDialect(dsTypeEnum).formatMethodFunction(column.getFunctionMethod(), columnName, columnName);
                //日期格式化查询
                }else if(column.getTimeDataFormat() != null) {
                    columnName = dialectContext.getDialect(dsTypeEnum).formatDateTime(column.getTimeDataFormat(), columnName);
                }
            }
            //判断columnName是否为空
            if (StrUtil.isEmpty(columnName)) {
                continue;
            }
            //计算过滤条件
            List<ComponentFilterCondition> conditions = columnFilter.getConditions();
            if (conditions != null && !conditions.isEmpty()) {
                //合并日期变量
                CubeConstants.mergeSysCurrentTimeParams(column.getTimeDataFormat(), extraParams);
                List<String> conditionSqlList = new ArrayList<String>();
                //添加过滤条件
                for (ComponentFilterCondition condition : conditions) {
                    //替换变量参数
                    if (StrUtil.isNotBlank(condition.getParamKey())) {
                        //当不需要替换全局变量值时：把"${key}"设置为value，数据查询过程中替换
                        condition.setValues(isReplaceGlobalParams ? extraParams.get(condition.getParamKey()) : ListUtil.toList("${"+condition.getParamKey()+"}"));
                    }
                    String conditionSql = condition.getFunction().handle(columnName, column.getViewDataType(), condition.getValues(), dsTypeEnum);
                    if (!StrUtil.isEmpty(conditionSql)) {
                        conditionSqlList.add(conditionSql);
                    }
                }
                if (!conditionSqlList.isEmpty()) {
                    //不同的过滤时机添加到对应的集合
                    if (connectTime == FilterConnectTimeEnum.HAVING) {
                        havingSqlList.add(_stringListJoin(conditionSqlList, columnFilter.getConnectType()));
                    } else {
                        whereSqlList.add(_stringListJoin(conditionSqlList, columnFilter.getConnectType()));
                    }
                }
            }
        }
        return new Tuple2<String, String>(_stringListJoin(whereSqlList, relation), _stringListJoin(havingSqlList, relation));
    }

    /**
     * SQL片段拼接
     *
     * @param dataList
     * @param relation
     * @return
     */
    private String _stringListJoin(List<String> dataList, FilterConnectTypeEnum relation) {
        if (dataList == null || dataList.isEmpty()) {
            return "";
        }
        String startParenthesis = dataList.size() > 1 ? "(" : "";
        String endParenthesis = dataList.size() > 1 ? ")" : "";
        return startParenthesis + String.join(" " + relation.name() + " ", dataList) + endParenthesis;
    }
}
