package org.springblade.visual.service;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.visual.entity.VisualComponentTemplate;
import org.springblade.visual.entity.VisualComponentTemplateCategory;
import org.springblade.visual.mapper.VisualComponentTemplateCategoryMapper;
import org.springblade.visual.mapper.VisualComponentTemplateMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;

@Lazy
@Service
public class VisualComponentTemplateCategoryService extends ServiceImpl<VisualComponentTemplateCategoryMapper, VisualComponentTemplateCategory> {

    @Autowired
    private VisualComponentTemplateMapper visualComponentTemplateMapper;

    /**
     * 创建目录
     */
    public boolean createCategory(VisualComponentTemplateCategory category) {
        category.setCreateTime(LocalDateTime.now())
                .setCreateUser(SecureUtil.getUserId());
        return save(category);
    }

    /**
     * 修改分类
     *
     * @param category
     */
    public boolean editCategory(VisualComponentTemplateCategory category) {
        if (category.getId() == null) {
            category.setCreateTime(LocalDateTime.now())
                    .setCreateUser(SecureUtil.getUserId());
            return save(category);
        } else {
            category.setUpdateTime(LocalDateTime.now())
                    .setUpdateUser(SecureUtil.getUserId());
            return updateById(category);
        }
    }

    /**
     * 删除分类
     */
    public boolean deleteCategory(Long id) {
        //判断是否有子分类
        List<VisualComponentTemplateCategory> list = this.lambdaQuery().eq(VisualComponentTemplateCategory::getParentId, id).list();
        if (list.size() > 0) {
            throw new ServiceException("请先删除子分类");
        }
        //判断目录下面是否有模板
        List<VisualComponentTemplate> templateList = visualComponentTemplateMapper.selectList(Wrappers.<VisualComponentTemplate>lambdaQuery().eq(VisualComponentTemplate::getCategory, id));
        if (CollectionUtil.isNotEmpty(templateList)) {
            throw new ServiceException("目录下存在组件模板");
        }
        return removeById(id);
    }

    /**
     * 查询所有目录
     * 然后通过查询到的数据的parent_id来构建一个树结构
     */
    public List<VisualComponentTemplateCategory> queryAllCategory(String name) {
        //查询所有目录
        List<VisualComponentTemplateCategory> categoryList = this.lambdaQuery().like(StrUtil.isNotBlank(name), VisualComponentTemplateCategory::getName, name).list();
        if (CollectionUtil.isEmpty(categoryList)) categoryList = Collections.emptyList();
        return categoryList;
    }

    /**
     * 查询所有目录(租户id是null的数据)
     */
    public List<VisualComponentTemplateCategory> queryMcpAllCategory(String name) {
        //查询所有目录(租户id是null的数据)
        List<VisualComponentTemplateCategory> categoryList = this.getBaseMapper().mcpCategoryList(name);
        if (CollectionUtil.isEmpty(categoryList)) categoryList = Collections.emptyList();
        return categoryList;
    }


}
