package org.springblade.visual.service;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.secure.utils.SecureUtil;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.CubeDataService;
import org.springblade.visual.dto.*;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.entity.VisualComponentTemplate;
import org.springblade.visual.entity.VisualComponentTemplateCategory;
import org.springblade.visual.entity.VisualReviewLog;
import org.springblade.visual.enums.ComponentUseTypeEnum;
import org.springblade.visual.mapper.VisualComponentTemplateMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

@Service
public class VisualComponentTemplateService extends ServiceImpl<VisualComponentTemplateMapper, VisualComponentTemplate> {

    @Autowired
    private QuerySqlService querySqlService;
    @Autowired
    private CubeDataService cubeDataService;
    @Autowired
    private VisualComponentService visualComponentService;
    @Autowired
    private VisualReviewLogService visualReviewLogService;
    @Autowired
    private VisualComponentTemplateCategoryService visualComponentTemplateCategoryService;

    public VisualComponentTemplate details(Long id) {
        VisualComponentTemplate componentTemplate = this.getById(id);
        if (componentTemplate != null) {
            componentTemplate.setVisualComponent(visualComponentService.getById(componentTemplate.getComponentId()));
        }
        return componentTemplate;
    }

    /**
     * 创建大屏模板
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public Integer submitComponentTemplate(SubmitComponentTemplateParams params) {
        VisualComponentTemplate visualComponentTemplate;
        if (params.getComponentTemplateId() != null) {
            visualComponentTemplate = baseMapper.selectById(params.getComponentTemplateId());
            if (visualComponentTemplate == null) {
                return null;
            }
            visualComponentTemplate.setUpdateUser(SecureUtil.getUserId());
            visualComponentTemplate.setUpdateTime(LocalDateTime.now());
        } else {
            visualComponentTemplate = new VisualComponentTemplate();
            visualComponentTemplate.setCreateUser(SecureUtil.getUserId());
            visualComponentTemplate.setCreateUserName(SecureUtil.getNickName());
            visualComponentTemplate.setCreateTime(LocalDateTime.now());
        }
        visualComponentTemplate.setCategory(params.getCategory());
        visualComponentTemplate.setName(params.getName());
        visualComponentTemplate.setPreviewUrl(params.getPreviewUrl());
        saveComponent(params.getComponentDto());
        //设置组件ID
        visualComponentTemplate.setComponentId(params.getComponentDto().getComponent().getId());
        if(visualComponentTemplate.getId() == null) {
            baseMapper.insert(visualComponentTemplate);
        }else {
            baseMapper.updateById(visualComponentTemplate);
        }
        return visualComponentTemplate.getId();
    }

    @Transactional(rollbackFor = Exception.class)
    public boolean copyComponentTemplate(Long id) {
        VisualComponentTemplate componentTemplate = baseMapper.selectById(id);
        VisualComponent visualComponent = visualComponentService.getById(componentTemplate.getComponentId());
        visualComponent.setId(null);
        visualComponentService.save(visualComponent);
        componentTemplate
                .setId(null)
                .setStatus(0)
                .setReviewStatus(0)
                .setComponentId(visualComponent.getId())
                .setName(componentTemplate.getName() + "-副本")
                .setCreateUser(AuthUtil.getUserId())
                .setCreateUserName(AuthUtil.getNickName())
                .setTenantId(AuthUtil.getTenantId());
        return baseMapper.insert(componentTemplate) > 0;
    }


    public void saveComponent(ComponentDto componentDto) {
        VisualComponent component = componentDto.getComponent();
        if (component.getId() == null) { //新增
            component.setCreateTime(new Date());
            component.setCreateBy(SecureUtil.getNickName());
            component.setCreateUserId(SecureUtil.getUserId());
        }
        component.setColumnJson(componentDto.getColumnList());
        component.setFilterJson(componentDto.getFilterList());
        component.setFilterRelationJson(componentDto.getFilterRelation());
        component.setSortJson(componentDto.getSortList());
        //当且仅当组件类型是Cube并且有字段信息解析SQL（有可能是个纯前端的组件或者是API类型组件）
        if (!StrUtil.isEmpty(componentDto.getColumnList())
                && component.getUseType() == ComponentUseTypeEnum.CUBE) {
            LambdaQueryWrapper<CubeData> queryWrapper = Wrappers.<CubeData>lambdaQuery().select(CubeData::getDatasourceId).eq(CubeData::getId, component.getCubeDataId());
            Integer datasourceId = cubeDataService.getObj(queryWrapper, t -> Integer.valueOf(t.toString()));
            //设置数据源ID
            component.setDatasourceId(datasourceId);

            //解析使用到的字段ID列表
//            Set<Integer> usedColumnIdList = querySqlService.parseUsedColumnIdList(componentDto);
//            //如果extJson中存在zuanList字段，也需要放到columnId列表中
//            if (StrUtil.isNotBlank(component.getExtJson())) {
//                try {
//                    JSONObject tempObject = JSON.parseObject(component.getExtJson());
//                    if (tempObject.containsKey("zuanList")) {
//                        JSON.parseArray(tempObject.getString("zuanList"), ComponentColumn.class).stream().map(ComponentColumn::getCubeColumnId).forEach(usedColumnIdList::add);
//                        ;
//                    }
//                } catch (Exception e) {
//                    log.error(e.getMessage(), e);
//                }
//            }
//            //设置使用到的字段ID列表JSON
//            component.setColumnIdList(JSON.toJSONString(usedColumnIdList));
        }
        visualComponentService.saveOrUpdate(component);
    }

    /**
     * 修改组件模板的基本信息
     *
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public boolean updateComponentTemplateBasicInfo(UpdateComponentTemplateBasicInfoParams params) {
        VisualComponentTemplate visualComponentTemplate = baseMapper.selectById(params.getId());
        if (visualComponentTemplate == null) {
            return false;
        }
        visualComponentTemplate.setName(params.getName())
                .setCategory(params.getCategory())
                .setPreviewUrl(params.getPreviewUrl())
                .setUpdateUser(SecureUtil.getUserId())
                .setUpdateTime(LocalDateTime.now());
        baseMapper.updateById(visualComponentTemplate);
        visualComponentService
                .lambdaUpdate()
                .eq(VisualComponent::getId, visualComponentTemplate.getComponentId())
                .set(VisualComponent::getName, params.getName())
                .update();
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    public boolean deleteComponentTemplate(Long id) {
        VisualComponentTemplate visualComponentTemplate = baseMapper.selectById(id);
        if (visualComponentTemplate == null) {
            return false;
        }
        baseMapper.deleteById(id);
        visualComponentService.removeById(visualComponentTemplate.getComponentId());
        return true;
    }

    /**
     * 分页查询
     *
     * @param params
     * @return
     */
    public IPage<VisualComponentTemplate> page(VisualComponentTemplatePageParams params) {
        Page<VisualComponentTemplate> page = this.lambdaQuery()
                .eq(params.getCategory() != null, VisualComponentTemplate::getCategory, params.getCategory())
                .like(StrUtil.isNotBlank(params.getName()), VisualComponentTemplate::getName, params.getName())
                .orderByDesc(VisualComponentTemplate::getCreateTime)
                .page(new Page<>(params.getPage(), params.getSize()));
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    public boolean submitReview(@RequestBody SubmitComponentTemplateReviewParams params) {
        VisualReviewLog visualReviewLog = new VisualReviewLog();
        visualReviewLog.setRemark(params.getReason())
                .setStatus(0)
                .setType(2)
                .setRelationId(params.getComponentTemplateId())
                .setCreateTime(LocalDateTime.now())
                .setCreateUserName(SecureUtil.getNickName())
                .setCreateUserId(SecureUtil.getUserId());
        visualReviewLogService.save(visualReviewLog);
        //修改审核状态
        this.lambdaUpdate()
                .eq(VisualComponentTemplate::getId, params.getComponentTemplateId())
                .set(VisualComponentTemplate::getReviewStatus, 1)
                .update();
        return true;
    }

    /**
     * 组件模板库-目录
     */
    public List<VisualComponentTemplateCategory> componentTemplateLibraryCategory(String name) {
        return visualComponentTemplateCategoryService.queryMcpAllCategory(name);
    }

    /**
     * 组件模板库-模板
     */
    public Page<VisualComponentTemplate> componentTemplateLibrary(VisualComponentTemplatePageParams params) {
        return baseMapper.allReviewedTemplate(new Page(params.getPage(), params.getSize()), params.getName(), params.getCategory());
    }

    @Async
    public void addUseTimes(Long componentTemplateId) {
        baseMapper.addUseTimes(componentTemplateId);
    }

    @Async
    public void addUseTimesBatch(List<Long> ids) {
        baseMapper.addUseTimesBatch(ids);
    }

}
