/*
 * Copyright (c) 2018-2028, aokunsang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.visual.service.component;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.cube.dto.PreviewResult;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.ComponentDto;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.service.QuerySqlService;
import org.springblade.visual.service.VisualComponentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * （自定义类描述）
 *
 * @author wookvn
 * @datetime 2024-10-12 11:31:27
 */
@Slf4j
@Component
public class VisualComponentSangjiService {

    @Autowired
    private VisualComponentService visualComponentService;

    @Autowired
    private DynamicTableService dynamicTableService;

    @Autowired
    private QuerySqlService querySqlService;

    /*
     * <p>（自定义方法描述）</p>
     **/
    public void previewData(Object obj, ComponentDto componentDto, List<PreviewResult> list, StringBuffer sqlps, BladeUser user, Integer order) {
        String filterColum = visualComponentService.getFilterColum(obj.toString());
        if (StrUtil.isNotEmpty(filterColum)) {
            List<Object> selectTimes = visualComponentService.getSelectTime(componentDto.getFilterList(), filterColum);
            if (selectTimes.size() < 1) {
                throw new ServiceException("请设置查询的时间范围...");
            }
            componentDto.setIsDateComplete(true);
            componentDto.setFilterColum(filterColum);
            componentDto.setSelectTime(selectTimes);
        }
        VisualComponent component = componentDto.getComponent();
        R<QuerySqlParam> r = querySqlService.parseQuerySql(component.getCubeDataId(), componentDto, user);
        if (!r.isSuccess()) {
            throw new ServiceException(r.getMsg());
        }
        List<Map<String, Object>> results;
        QuerySqlParam querySqlParam = r.getData();
        ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
        String querySql = querySqlParam.getQuerySql();
        List<ComponentColumn> selectColumnList = querySqlParam.getSelectColumnList();
        Map<String, String> fieldDict = selectColumnList.stream().collect(Collectors.toMap(ComponentColumn::getColumnAliasName, ComponentColumn::getColumnTitle));
        List<Map<String, Object>> resultList = null;
        if (component.getLimitVal() != null && component.getLimitVal() > 0) {
            resultList = dynamicTableService.selectPageList(new Page<>(0, component.getLimitVal()).setSearchCount(false),
                    reportDatasource, querySql);
        } else {
            resultList = dynamicTableService.selectList(reportDatasource, querySql);
        }
        //列权限，删除掉不显示的列
        visualComponentService.postPreviewData(fieldDict, resultList, selectColumnList, querySqlParam.getExcludeColumnIdList(), querySqlParam);
        String cubeName = querySqlParam.getCubeData() != null ? querySqlParam.getCubeData().getCubeName() : null;
        list.add(new PreviewResult(resultList, fieldDict, cubeName, order));
        sqlps.append(querySql).append(";");
    }


}
