/*
 * Copyright (c) 2018-2028, aokunsang All rights reserved.
 * 我问青山何时老，青山问我几时闲，
 * 不是闲人闲不得，能闲必非等闲人。
 */
package org.springblade.visual.service.component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Stack;
import java.util.stream.Collectors;

import org.springblade.core.secure.BladeUser;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.exception.ServiceException;
import org.springblade.core.tool.utils.CollectionUtil;
import org.springblade.cube.dto.wayneDiagramResult;
import org.springblade.cube.model.CubeData;
import org.springblade.cube.service.DynamicTableService;
import org.springblade.cube.utils.CubeUtils;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.visual.dto.ComponentColumn;
import org.springblade.visual.dto.ComponentDto;
import org.springblade.visual.dto.ComponentSort;
import org.springblade.visual.dto.PreviewBaseDto;
import org.springblade.visual.dto.QuerySqlParam;
import org.springblade.visual.entity.VisualComponent;
import org.springblade.visual.service.QuerySqlService;
import org.springblade.visual.service.VisualComponentService;
import org.springblade.visual.utils.DateSqlUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson2.JSON;

import cn.hutool.core.lang.Tuple;
import cn.hutool.core.util.StrUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * （自定义类描述）
 *
 * @author wookvn
 * @datetime 2024-10-12 11:39:24
 */
@Slf4j
@Component
public class VisualComponentWeienService {

    @Autowired
    private VisualComponentService visualComponentService;

    @Autowired
    private DynamicTableService dynamicTableService;

    @Autowired
    private QuerySqlService querySqlService;


    public void previewWayneDiagram(String base64Value, List<wayneDiagramResult> list, StringBuffer sqlps) {
        //前置校验
        ComponentDto componentDto = visualComponentService.previewCheck(base64Value);
        //1.先根据维度数据聚合查询分页记录
        VisualComponent component = componentDto.getComponent();
        BladeUser blauser = AuthUtil.getUser();
        R<QuerySqlParam> r = querySqlService.parseQuerySql(component.getCubeDataId(), componentDto, blauser);
        if (!r.isSuccess()) {
            throw new ServiceException(r.getMsg());
        }
        QuerySqlParam querySqlParam = r.getData();
        ReportDatasource reportDatasource = querySqlParam.getReportDatasource();
        CubeData cubeData = querySqlParam.getCubeData();
        String sql = splicingSql(componentDto, blauser, reportDatasource, cubeData);
        sqlps.append(sql);
        List<Map<String, Object>> mapList = dynamicTableService.selectList(reportDatasource, sql);
        if (CollectionUtil.isEmpty(mapList)) {
            return;
        }
        List<Map<String, Object>> resultList = new LinkedList<>();
        for (Map<String, Object> map : mapList) {
            String colAlias0 = map.get("COL_ALIAS_0").toString();
            List<String> distinct = Arrays.asList(colAlias0.split(",")).stream().distinct().collect(Collectors.toList());
            map.put("COL_ALIAS_0", String.join(",", distinct));
            resultList.add(map);
        }
        List<Map<String, Object>> fileterList = new ArrayList<>();
        resultList.forEach(map -> {
            if (Arrays.asList(map.get("COL_ALIAS_0").toString().split(",")).size() == 1) {
                fileterList.add(map);
            }
        });
        // 默认限制的条数
        List<Map<String, Object>> limits;
        // 如果限制条数不为空则限制查询出来的条数，按照从大到小排序
        if (component.getLimitVal() != null && component.getLimitVal() > 0) {
            limits = new ArrayList<>(fileterList.stream().limit(component.getLimitVal()).collect(Collectors.toList()));
        } else {
            // 如果限制条数为空则表示全量查询展示
            for (Map<String, Object> map : resultList) {
                List<String> colAlias = Arrays.asList(map.get("COL_ALIAS_0").toString().split(","));
                List<String> collect = colAlias.stream().distinct().collect(Collectors.toList());
                // 结果转换
                wayneDiagramResult result = new wayneDiagramResult();
                result.setLabel(String.join(",", collect));
                result.setSize(map.get("COL_ALIAS_1").toString());
                result.setSets(collect);
                list.add(result);
            }
            return;
        }

        // 对过滤的条件进行排列组合
        List<String> columns = permutationAndCombination(limits);
        // 对查询结果进行条数限制
        for (Map<String, Object> map : resultList) {
            List<String> colAlias = Arrays.asList(map.get("COL_ALIAS_0").toString().split(","));
            List<String> collect = colAlias.stream().distinct().collect(Collectors.toList());
            for (String column : columns) {
                List<String> strings = Arrays.asList(column.split(","));
                strings.sort(String::compareTo);
                collect.sort(String::compareTo);
                if (String.join(",", strings).equals(String.join(",", collect))) {
                    // 结果转换
                    wayneDiagramResult result = new wayneDiagramResult();
                    result.setLabel(String.join(",", collect));
                    result.setSize(map.get("COL_ALIAS_1").toString());
                    result.setSets(collect);
                    list.add(result);
                }
            }
        }
        for (Map<String, Object> limit : limits) {
            List<String> colAlias = Arrays.asList(limit.get("COL_ALIAS_0").toString().split(","));
            // 结果转换
            wayneDiagramResult result = new wayneDiagramResult();
            result.setLabel(String.join(",", colAlias));
            result.setSize(limit.get("COL_ALIAS_1").toString());
            result.setSets(colAlias);
            list.add(result);
        }
    }

    /**
     * 对过滤条件进行排列组合
     */
    private List<String> permutationAndCombination(List<Map<String, Object>> limits) {
        List<String> columns = new ArrayList<>();
        List<String> columnTitles = new ArrayList<>();
        for (Map<String, Object> map : limits) {
            columnTitles.add(map.get("COL_ALIAS_0").toString());
        }
        for (int i = 1; i < columnTitles.size(); i++) {
            _f(columnTitles, i + 1, 0, 0, columns);
        }
        return columns;
    }

    public static Stack<String> stack = new Stack<>();

    /**
     * @param shu  元素
     * @param targ 要选多少个元素
     * @param has  当前有多少个元素
     * @param cur  当前选到的下标
     *             <p>
     *             1    2   3     //开始下标到2
     *             1    2   4     //然后从3开始
     */
    private void _f(List<String> shu, int targ, int has, int cur, List<String> result) {
        if (has == targ) {
            StringBuffer mgs = new StringBuffer();
            for (String s : stack) {
                mgs.append(s).append(",");
            }
            result.add(mgs.substring(0, mgs.length() - 1));
            return;
        }

        for (int i = cur; i < shu.size(); i++) {
            if (!stack.contains(shu.get(i))) {
                stack.add(shu.get(i));
                _f(shu, targ, has + 1, i, result);
                stack.pop();
            }
        }
    }

    /**
     * 拼接桑基图sql
     */
    private String splicingSql(PreviewBaseDto previewDto, BladeUser user, ReportDatasource reportDatasource, CubeData cubeData) {
        String sql = "select b as COL_ALIAS_0, count(a) as COL_ALIAS_1 from ( $sql ) S group by b order by COL_ALIAS_1 desc";
        DsTypeEnum dsTypeEnum = reportDatasource.getDsTypeSyl();
        // 根据不同的sql拼接不同的韦恩图查询sql
        String strBuilder = DateSqlUtils.getSqlByDsType(dsTypeEnum);
        //构造全局变量
        Map<String, List<Object>> extraParams = querySqlService.buildGlobalParams(previewDto.getExtraParams(), user);

        //2.构造Select模块
        List<ComponentColumn> columnList = JSON.parseArray(previewDto.getColumnList(), ComponentColumn.class);
        Map<Integer, ComponentColumn> columnMap = columnList.stream().collect(Collectors.toMap(ComponentColumn::getCubeColumnId, t -> t, (v1, v2) -> v1));
        //主要用于排序时，不能使用带有%的字符串
        Map<Integer, String> originalNumberColumnMap = new HashMap<>();
        int num = 0;
        String tableAliasName = "";
        for (ComponentColumn column : columnList) {
            // 替换字段
            strBuilder = strBuilder.replace("$table", cubeData.getQuerySql()).replace("col_" + num, column.getColumnSql());
            tableAliasName = column.getTableAliasName();
            num++;
        }
        //解析where和having子句
        Tuple whereHavingSql = querySqlService.parseWhereHavingSql(cubeData, dsTypeEnum, previewDto, user, extraParams);
        String whereSql = whereHavingSql.get(0);
        if (StrUtil.isNotBlank(whereSql)) {
            strBuilder = strBuilder.replace("$where", "where " + whereSql);
        } else {
            strBuilder = strBuilder.replace("$where", " ");
        }
        //构造排序条件
        List<ComponentSort> sortList = Optional.ofNullable(JSON.parseArray(previewDto.getSortList(), ComponentSort.class)).orElse(new ArrayList<>());
        List<String> sortSqlList = new ArrayList<String>();
        for (ComponentSort sort : sortList) {
            ComponentColumn column = sort.getColumn();
            //获取别名
            ComponentColumn componentColumn = columnMap.get(column.getCubeColumnId());
            if (componentColumn != null) { //如果别名不在select中，不能放到orderBy中
                //如果排序字段带有%，使用原始字段
                String orgialColumnName = originalNumberColumnMap.get(column.getCubeColumnId());
                String columnName = StrUtil.isNotBlank(orgialColumnName) ? orgialColumnName : componentColumn.getColumnAliasName();
                sortSqlList.add(columnName + " " + sort.getOrderType().name());
            }
        }
        if (!sortSqlList.isEmpty()) {
            strBuilder = strBuilder + " order by " + (String.join(",", sortSqlList));
        }
        if (CubeUtils.checkSqlInject(strBuilder.toString())) {
            log.info("sqlInject:warning|sql=" + strBuilder);
            return null;
        }
        if (DsTypeEnum.oracle.getValue().equals(dsTypeEnum.getValue())) {
            sql = "select dbms_lob.substr(b,2000) as COL_ALIAS_0, count(a) as COL_ALIAS_1 from ( $sql ) S group by dbms_lob.substr(b,2000) order by COL_ALIAS_1 desc";
        }
        return sql.replace("$sql", strBuilder);
    }

}
