package org.springblade.visual.utils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.report.entity.ReportDatasource;
import org.springblade.visual.dto.ComponentColumn;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.NumberUtil;

public class DateSqlUtils {

    public static String getNormalSqlByDatasource(ReportDatasource reportDatasource) {
        String sql = "SELECT distinct concat(a00.YYYYMMDD,case when a00.hour < 10 then concat('0',a00.hour) else concat('',hour) end) AS ${0}, ${1}\n" +
                "FROM ${2} a00\n" +
                "         LEFT JOIN (${3}) b00 \n" +
                "                   ON concat(a00.yyyyMMdd,case when a00.hour < 10 then concat('0',a00.hour) else concat('',a00.hour) end) = b00.${0} \n" +
                "WHERE concat(a00.YYYYMMDD,case when a00.hour < 10 then concat('0',a00.hour) else concat('',hour) end) between ${4} and ${5} \n" +
                "ORDER BY concat(a00.YYYYMMDD,case when a00.hour < 10 then concat('0',a00.hour) else concat('',hour) end),${1}";
        return sql;
    }

    public static String getHourSqlByDatasource(ReportDatasource reportDatasource) {
        // 默认为mysql
        String sql = "SELECT distinct concat(a00.YYYYMMDD,case when a00.hour < 10 then concat('0',a00.hour) else concat('',a00.hour) end) AS ${0},${1}\n" +
                "FROM ${2} a00\n" +
                "         LEFT JOIN (${3}) b00\n" +
                "                   ON a00.HOUR = SUBSTRING_INDEX(SUBSTRING_INDEX(${0}, ':', 1),' ',-1)\n" +
                "WHERE a00.HOUR BETWEEN 0 AND 23\n" +
                "ORDER BY a00.YYYYMMDD,a00.HOUR";
        if (reportDatasource.getDsType() == 1) { //oracle
            // SELECT concat(concat(concat(a00.YYYYMMDD, ' '), a00.HOUR), ':00:00') AS ${0}, ${1}
            sql = "SELECT distinct a00.YYYYMMDD AS ${0},a00.HOUR AS HOUR, ${1}\n" +
                    "FROM ${2} a00\n" +
                    "         LEFT JOIN (${3}) b00\n" +
                    "                   ON a00.HOUR = substr(substr(${0}, 1, instr(${0}, ':') - 1),\n" +
                    "              instr(substr(${0}, 1, instr(${0}, ':') - 1),' '))\n" +
                    "WHERE a00.HOUR BETWEEN 0 AND 23\n" +
                    "ORDER BY a00.YYYYMMDD,a00.HOUR";
        } else if (reportDatasource.getDsType() == 2) { //sqlserver

        } else if (reportDatasource.getDsType() == 3) { //postgresql

        } else if (reportDatasource.getDsType() == 4) { //clickhouse

        } else if (reportDatasource.getDsType() == 5) { //kingbase
            sql = "SELECT distinct concat(a00.YYYYMMDD,case when a00.hour < 10 then concat('0',a00.hour) else concat('',a00.hour) end) AS ${0},${1}\n" +
                    "FROM ${2} a00\n" +
                    "         LEFT JOIN (${3}) b00\n" +
                    "                   ON a00.HOUR = SPLIT_PART(SPLIT_PART(${0}, ':', 1),' ',2)\n" +
                    "WHERE a00.HOUR BETWEEN 0 AND 23\n" +
                    "ORDER BY a00.YYYYMMDD,concat(a00.YYYYMMDD,case when a00.hour < 10 then concat('0',a00.hour) else concat('',a00.hour) end),a00.HOUR,${1}";
        } else if (reportDatasource.getDsType() == 6) {

        } else if (reportDatasource.getDsType() == 7) {

        } else if (reportDatasource.getDsType() == 8) {

        }

        return sql;
    }

    public static String getSqlByDatasource(ReportDatasource reportDatasource) {
        // 默认为mysql
        String sql = "SELECT distinct a00.${0} AS ${1}, ${2} \n" +
                "FROM tableName a00\n" +
                " LEFT JOIN (${3}) b00 ON a00.${0} = REPLACE(REPLACE(b00.${1}, '-', ''),'/','')\n" +
                "WHERE a00.${0} BETWEEN ${4} AND ${5} \n" +
                "ORDER BY a00.${0}";
        if (reportDatasource.getDsType() == 1) { //oracle
            sql = "SELECT distinct a00.${0} AS ${1}, ${2} \n" +
                    "FROM tableName a00\n" +
                    " LEFT JOIN (${3}) b00 ON a00.${0} = to_number(REPLACE(REPLACE(b00.${1}, '-', ''),'/',''))\n" +
                    "WHERE a00.${0} BETWEEN ${4} AND ${5} \n" +
                    "ORDER BY a00.${0}";
        } else if (reportDatasource.getDsType() == 2) { //sqlserver
            sql = "SELECT distinct a00.${0} AS ${1}, ${2} \n" +
                    "FROM tableName a00\n" +
                    " LEFT JOIN (${3}) b00 ON a00.${0} = REPLACE(REPLACE(b00.${1}, '-', ''),'/','')\n" +
                    "WHERE a00.${0} BETWEEN ${4} AND ${5} \n" +
                    "ORDER BY a00.${0}";
        } else if (reportDatasource.getDsType() == 3) { //postgresql
            sql = "SELECT distinct a00.${0} AS ${1}, ${2} \n" +
                    "FROM tableName a00\n" +
                    " LEFT JOIN (${3}) b00 ON a00.${0} = REPLACE(REPLACE(b00.${1}, '-', ''),'/','')\n" +
                    "WHERE a00.${0} BETWEEN ${4} AND ${5} \n" +
                    "ORDER BY a00.${0}";
        } else if (reportDatasource.getDsType() == 4) { //clickhouse
            sql = "SELECT distinct a00.${0} AS ${1}, ${2} \n" +
                    "FROM tableName a00\n" +
                    " LEFT JOIN (${3}) b00 ON a00.${0} = REPLACE(REPLACE(b00.${1}, '-', ''),'/','')\n" +
                    "WHERE a00.${0} BETWEEN ${4} AND ${5} \n" +
                    "ORDER BY a00.${0}";
        } else if (reportDatasource.getDsType() == 5) { //kingbase
            sql = "SELECT distinct a00.${0} AS ${1}, ${2} \n" +
                    "FROM tableName a00\n" +
                    " LEFT JOIN (${3}) b00 ON a00.${0} = REPLACE(REPLACE(b00.${1}, '-', ''),'/','')\n" +
                    "WHERE a00.${0} BETWEEN ${4} AND ${5} \n" +
                    "ORDER BY a00.${0},${2}";
        } else if (reportDatasource.getDsType() == 6) {

        } else if (reportDatasource.getDsType() == 7) {

        } else if (reportDatasource.getDsType() == 8) {

        }

        return sql;
    }

    public static String getSqlByDsType(DsTypeEnum dsTypeEnum) {
        // 默认为mysql
        String strBuilder = "select col_1 as a, group_concat(distinct col_0)  as b from $table $where group by col_1";
        if (DsTypeEnum.oracle.getValue().equals(dsTypeEnum.getValue())) { // oracle
//            strBuilder = "SELECT col_1 as a, LISTAGG(col_0, ',') WITHIN GROUP (ORDER BY col_0) AS b FROM $table $where  GROUP BY col_1";
            strBuilder = "SELECT col_1 as a, xmlagg(xmlparse(content col_0 || ',' wellformed) order by col_0).getclobval() AS b " +
                    "FROM $table $where  GROUP BY col_1";
        } else if (DsTypeEnum.sqlserver.getValue().equals(dsTypeEnum.getValue())) { // sqlserver
            strBuilder = "SELECT T.COL_0 as a,\n" +
                    "       STUFF((SELECT ',' + A.col_1\n" +
                    "              FROM $table A\n" +
                    "              WHERE A.col_0 = T.COL_0\n" +
                    "              group by A.COL_1\n" +
                    "              FOR XML PATH ( '' )), 1, 1, '') AS b\n" +
                    "FROM $table T\n" +
                    "$where\n" +
                    "GROUP BY T.COL_0";
        } else if (DsTypeEnum.postgresql.getValue().equals(dsTypeEnum.getValue())) { // pg
            strBuilder = "SELECT COL_0 as a, string_agg(distinct COL_1,',') AS b FROM $table $where GROUP BY COL_0";
        } else if (DsTypeEnum.clickhouse.getValue().equals(dsTypeEnum.getValue())) { // clickhouse
            strBuilder = "SELECT COL_0 as a, arrayStringConcat(groupArray(COL_1), ',') AS b FROM $table $where GROUP BY COL_0";
        } else if (DsTypeEnum.kingbase.getValue().equals(dsTypeEnum.getValue())) { // 人大金仓
            strBuilder = "SELECT col_0 as a, GROUP_CONCAT(col_1) as b FROM $table $where GROUP BY col_1";
        }
        return strBuilder;
    }

    public static Map<String, Object> getSql(ComponentColumn dataColum, List<Object> selectTime, String joinSql, List<ComponentColumn> columnList, ReportDatasource reportDatasource) {
        Map<String, Object> map = new HashMap<>();
        String table = "dim_date";
        String dateTimeColum = "YEAR";
        // 判断sql的时间参数是否为数字时间
        boolean bool = isNumberTime(selectTime);
        String sql = DateSqlUtils.getSqlByDatasource(reportDatasource);
        // 常规时间类型的处理方式
        if (!bool) {
            if (dataColum.getTimeDataFormat() != null) {
                // 判断使用哪个表
                if (dataColum.getTimeDataFormat().getValue() > 7) {
                    table = "dim_hour";
                    sql = DateSqlUtils.getHourSqlByDatasource(reportDatasource);
                }
                if (dataColum.getTimeDataFormat().getValue() >= 2 && dataColum.getTimeDataFormat().getValue() <= 4) {
                    dateTimeColum = "YEARMONTHID";
                } else if (dataColum.getTimeDataFormat().getValue() >= 5 && dataColum.getTimeDataFormat().getValue() <= 7) {
                    dateTimeColum = "YYYYMMDD";
                } else if (dataColum.getTimeDataFormat().getValue() >= 8) {
                    dateTimeColum = "YYYYMMDD";
                }
            }
        }

        // 获取字段的查询列表
        StringBuilder cloumStrs = new StringBuilder();
        for (ComponentColumn column : columnList) {
            if (!column.getColumnName().equals(dataColum.getColumnName())) {
                cloumStrs.append("b00.").append(column.getColumnAliasName()).append(",");
            }
        }

        // 获取sql的查询参数(分两种情况，一种情况是过滤器的时间范围，一种是过滤字段的时间范围)
        List<String> selectTimeList = new ArrayList<>();
        List<String> dataTimes = new ArrayList<>();
        if (selectTime.size() > 1) {
            for (Object o : selectTime) {
                // 如果当前的时间包含小时
                if (dataColum.getTimeDataFormat().getValue() >= 8) {
                    // 时间包含年月日小时
                    if (dataColum.getTimeDataFormat().getValue() >= 8 && dataColum.getTimeDataFormat().getValue() <= 10) {
                        if (o.toString().contains(":")) {
                            String[] split = o.toString().split(" ");
                            dataTimes.add(split[0].replace("[", "").replace("]", "")
                                    .replace("-", "").replace("/", ""));
                            String[] strings = split[1].split(":");
                            selectTimeList.add(Integer.valueOf(strings[0]).toString());
                        } else {
                            if (o.toString().length() > 2) {
                                selectTimeList.add(o.toString().trim().replace("[", "").replace("]", ""));
                            } else {
                                selectTimeList.add("0");
                            }
                        }
                    } else {
                        // 时间只包含小时
                        String[] split = o.toString().split(":");
                        selectTimeList.add(split[0]);
                    }
                } else {
                    // 当前时间为年月日
                    selectTimeList.add(o.toString().replace("[", "").replace("]", "")
                            .replace("-", "").replace("/", ""));
                }
            }
        } else {
            List<Object> objs = (List<Object>) selectTime.get(0);
            for (Object str : objs) {
                // 如果当前的时间包含小时
                if (dataColum.getTimeDataFormat().getValue() >= 8) {
                    if (dataColum.getTimeDataFormat().getValue() >= 8 && dataColum.getTimeDataFormat().getValue() <= 10) {
                        if (str.toString().contains(":")) {
                            String[] split = str.toString().split(" ");
                            if (split.length > 2) {
                                dataTimes.add(split[1].replace("[", "").replace("]", "")
                                        .replace("-", "").replace("/", ""));
                                String[] strings = split[2].split(":");
                                selectTimeList.add(Integer.valueOf(strings[0]).toString());
                            } else {
                                dataTimes.add(split[0].replace("[", "").replace("]", "")
                                        .replace("-", "").replace("/", ""));
                                String[] strings = split[1].split(":");
                                selectTimeList.add(Integer.valueOf(strings[0]).toString());
                            }
                        } else {
                            if (str.toString().length() > 2) {
                                selectTimeList.add(str.toString().trim().replace("[", "").replace("]", ""));
                            } else {
                                selectTimeList.add("0");
                            }
                        }
                    } else {
                        String[] split = str.toString().split(":");
                        selectTimeList.add(split[0]);
                    }
                } else {
                    // 当前时间为年月日
                    selectTimeList.add(str.toString().replace("[", "").replace("]", "")
                            .replace("-", "").replace("/", ""));
                }

            }
        }
        String s = selectTimeList.get(0);
        String s1 = selectTimeList.get(1);
        // 判断数字时间格式的查询列表名以及查询sql
        if (bool) {
            if (s.length() == 6 && (!s.contains("-") || !s.contains("/"))) {
                // 202306
                dateTimeColum = "YEARMONTHID";
            } else if (s.length() == 8 && (!s.contains("-") || !s.contains("/"))) {
                // 20230627
                dateTimeColum = "YYYYMMDD";
            } else if ((!s.contains("-") || !s.contains("/"))) {
                // 2023062715
                if (s.length() == 9 || s.length() == 10) {
                    dateTimeColum = "YYYYMMDD";
                    table = "dim_hour";
                    sql = DateSqlUtils.getNormalSqlByDatasource(reportDatasource);
                }
            }
        }
        String resultSql = "";
        if (bool && s.length() > 8) {
            resultSql = sql.replace("${0}", dataColum.getColumnAliasName())
                    .replace("${1}", cloumStrs.toString().substring(0, cloumStrs.length() - 1))
                    .replace("${2}", table)
                    .replace("${3}", joinSql)
                    .replace("${4}", s)
                    .replace("${5}", s1);
        } else {
            if (table.equals("dim_hour")) {
                resultSql = sql.replace("${0}", dataColum.getColumnAliasName())
                        .replace("${1}", cloumStrs.toString().substring(0, cloumStrs.length() - 1))
                        .replace("${2}", table)
                        .replace("${3}", joinSql);
                if (CollectionUtil.isNotEmpty(dataTimes)) {
                    String selectSql = " and a00.YYYYMMDD BETWEEN " + dataTimes.get(0) + " and " + dataTimes.get(1) + " ORDER BY";
                    resultSql = resultSql.replace("ORDER BY", selectSql);
                    map.put("dataTime", dataTimes);
                }
                map.put("timeColum", dataColum.getColumnAliasName());
                map.put("hourTime", selectTimeList);
            } else {
                resultSql = sql.replace("${0}", dateTimeColum)
                        .replace("${1}", dataColum.getColumnAliasName())
                        .replace("${2}", cloumStrs.toString().substring(0, cloumStrs.length() - 1))
                        .replace("${3}", joinSql)
                        .replace("${4}", s)
                        .replace("${5}", s1)
                        .replace("tableName", table);
            }
        }
        map.put("sql", resultSql);
        return map;
    }

    /**
     * 判断时间参数是否为数字日期
     */
    private static boolean isNumberTime(List<Object> selectTime) {
        if (selectTime.size() > 1) {
            for (Object o : selectTime) {
                boolean numeric = NumberUtil.isNumber(o.toString().replace("[", "").replace("]", ""));
                if (!numeric) {
                    return false;
                }
            }
        } else {
            List<Object> objs = (List<Object>) selectTime.get(0);
            for (Object obj : objs) {
                boolean numeric = NumberUtil.isNumber(obj.toString().replace("[", "").replace("]", ""));
                if (!numeric) {
                    return false;
                }
            }
        }
        return true;
    }
}
